/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.army.entities;

import fb.battlefield.Sprite;
import fb.battlefieldelements.Warrior;
import fb.util.Resources;

/**
 * unit using war machines
 * @author farfpeuf
 *
 */
public class Crew extends Shootingunit {

	/**
	 * the mahines
	 */
	private WarMachine machines;

	/**
	 * is the unit attached to its machines
	 */
	private boolean detached;
	/**
	 * create unit from line read in catalog
	 * 
	 * @param ligne
	 *            catalog line
	 * @param linenumber
	 *            line number
	 */
	public Crew(String ligne, int linenumber) {
		super(ligne, linenumber);
		machines = new WarMachine(distanceWeapon, this);
		projectilesNumber = 6;
	}
	/**
	 * calculate unit cost
	 */
	public int getCost() {
		return super.getCost() + machines.cost();
	}
/**
 * get machine name
 */
	public String distanceWeaponString() {
		if (distanceWeapon == 'c') {
			return Resources.getString("catapult");
		} else if (distanceWeapon == 'g') {
			return Resources.getString("giantcrossbow");
		} else if (distanceWeapon == 't') {
			return Resources.getString("trebuchet");
		} else
			return "-";
	}
	/**
	 * get this unit movement points
	 * 
	 * @return this unit movement points
	 */
	public int getMovement() {
		if (detached) {
			return super.getMovement();
		}
		return machines.getMovement();
	}
/**
 * amunitions left
 */
	public int getProjectilesNumber() {
		if (detached) {
			return 0;
		}
		return super.getProjectilesNumber();
	}
	/**
	 * create all unit warriors
	 * 
	 */
	public void createModel() {
		super.createModel();
		if (!detached) {
			machines.createModel();
		}
	}
	/**
	 * create entity sprite
	 * 
	 */
	public void createSprite() {
		super.createSprite();
		if (!detached) {
			machines.createsprite();
		}
	}
	/**
	 * delete sprite to free memory
	 * 
	 */
	public void freeSprite() {
		super.freeSprite();
		machines.freeSprite();
	}
	/**
	 * initialize unit at the beginning of a battle
	 * 
	 */
	public void initBattle() {
		super.initBattle();
		if (!detached) {
			machines.initBattle();
		}
	}
	/**
	 * initialize front line at the beginning of a battle
	 * 
	 */
	public void initFront() {
		setFront(getNumber());
	}
	/**
	 * is this unit shooting
	 * 
	 * @return true if one of the machines of this unit is shooting
	 */
	public boolean isShooting() {
		if (detached) {
			return false;
		}
		return machines.isShooting();
	}
	/**
	 * get number of frames in entity sprite
	 */
	public int getNumberOfFrames() {
		return 120;
	}
	/**
	 * add shooting sequences to sprite
	 */
	public void addSpriteShootingSequences(Sprite sprite) {
	}
	/**
	 * get entity shooting worth
	 * 
	 * @return entity shooting worth
	 */
	public int shootingWorth() {
		return machines.getWorth() * machines.getNumber() * volleys;
	}
	/**
	 * get entity shooting strength
	 * 
	 * @return entity shooting strength
	 */
	public int shootingStrength() {
		return machines.getStrength();
	}
	/**
	 * score needed to kill target
	 * @param shooterWeapon shooter weapon
	 * @param targetArmor target armor
	 * @return score needed to kill target
	 */
	public int shootToKill(char shooterWeapon, char targetArmor) {
		return machines.shootToKill(shooterWeapon, targetArmor);
	}
	/**
	 * start shooting animations
	 * 
	 */
	public void animShoot(Entity target) {
		double angledeshoot = angleTo(target);
		machines.animShoot(target, angledeshoot);
	}
	/**
	 * reset unit after end of battle
	 * 
	 * @param winner
	 *            did the unit army won the battle
	 */
	public void endBattle(boolean winner) {
		super.endBattle(winner);
		projectilesNumber = 6;
		detached = false;
	}
	/**
	 * @return Returns the midRange.
	 */
	public int getMidRange() {
		return machines.getMidRange();
	}
	/**
	 * @return Returns the longRange.
	 */
	public int getLongRange() {
		return machines.getLongRange();
	}
	/**
	 * @return Returns the shortRange.
	 */
	public int getShortRange() {
		return machines.getShortRange();
	}
	/**
	 * @return Returns the shootingRows.
	 */
	public int getShootingrows() {
		return machines.getShootingRows();
	}
	/**
	 * clone this entity
	 */
	public Object clone() {
		Crew clone = (Crew) super.clone();
		clone.machines = (WarMachine) machines.clone(clone);
		return clone;
	}
/**
 * initialize number
 */
	protected void initNumber() {
		setNumber(2);
	}
	/**
	 * @return Returns the max number of warriors.
	 */
	public int getMax() {
		return machines.getMax() * machines.getNormalCrewNumber();
	}
	/**
	 * @return Returns the min number of warriors.
	 */
	public int getMin() {
		return machines.getMin() * machines.getNormalCrewNumber();
	}
	/**
	 * set initial number of wariors
	 * 
	 * @param initialnumber
	 *            new initial number of wariors
	 */
	public void setInitialNumber(int init) {
		machines.setInitialNumber(init / machines.getNormalCrewNumber());
		super.setInitialNumber(init);
		if (machines.getNumber() <= 0) {
			setDetached(true);
		}
	}
	/**
	 * load unit from a StringBuffer
	 * 
	 * @param buffy
	 *            the StringBuffer
	 */
	public void loadString(StringBuffer buffy) {
		super.loadString(buffy);
		machines.setInitialNumber(
			getInitialNumber() / machines.getNormalCrewNumber());
		machines.setNumber(getNumber() / machines.getNormalCrewNumber());
		if (machines.getNumber() <= 0 || projectilesNumber <= 0) {
			setDetached(true);
		}
	}
/**
 * spend volleys used for shotting
 */
	public void spendVolleys() {
		super.spendVolleys();
		if (projectilesNumber <= 0) {
			machines.commitKill(0);
			setDetached(true);
		}
	}
	/**
	 * @return Returns the price.
	 */
	public int getPrice() {
		return super.getPrice()
			+ machines.getPrice() / machines.getNormalCrewNumber();
	}
	/**
	 * get entity strength as a string
	 * 
	 * @return entity strength as a string
	 */
	public String strengthString() {
		return Integer.toString(getStrength() + 1)
			+ "&"
			+ Integer.toString(machines.getStrength() + 1);
	}
	/**
	 * get entity worth as a string
	 * 
	 * @return entity worth as a string
	 */
	public String worthString() {
		return Integer.toString(getWorth())
			+ "&"
			+ Integer.toString(machines.getWorth() + 1);
	}
	/**
	 * remove dead warriors after killing
	 * 
	 */
	public void commitKill() {
		super.commitKill();
		if (!detached) {
			int nbMachines = getNumber() / machines.getNormalCrewNumber();
			machines.commitKill(nbMachines);
			if (nbMachines <= 0) {
				setDetached(true);
			} else {
				machines.organize();
				machines.orientate(getOrientation(), getX(), getY());
				machines.animMovement();
			}
		}

	}
	/**
	 * remove this unit after routing
	 * 
	 */
	public void commitRout() {
		super.commitRout();
		machines.commitRout();
	}
	/**
	 * get row length
	 * 
	 * @return row length
	 */
	public float rowLength() {
		if (detached) {
			return super.rowLength();
		} else {
			return machines.getLength() + getLength();
		}
	}
	/**
	 * organize warriors positions
	 */
	public void organize() {
		if (detached) {
			super.organize();
		} else {
			nbRows = 0;
			int reste = getNumber();
			int xd, yd, line;
			for (int i = 0; i < getNumber();) {
				if (reste > getFront()) {
					line = getFront();
				} else {
					line = reste;
				}
				for (int j = 0; j < line; j++) {
					xd =
						(int) ((((float) j - (float) (line - 1) / 2)
							* getWidth()
							* 8)
							+ getX());
					if (getFront() % 2 == 0) {
						xd -= (getWidth() * 8 / 2);
					}
					yd =
						(int) ((nbRows
							* (getLength() + machines.getLength())
							* 8)
							+ getY()
							- machines.getLength() * 8 / 2);
					((Warrior) models.get(i)).moveTo(xd, yd);
					i++;
				}
				nbRows++;
				reste -= getFront();
			}
			organizeSigns();
			machines.organize();
		}

	}
	/**
	 * start movement animation
	 * 
	 */
	public void animMovement() {
		super.animMovement();
		if (!detached) {
			machines.animMovement();
		}

	}
	/**
	 * orientate all the warriors
	 * 
	 */
	public void orientate() {
		super.orientate();
		if (!detached) {
			machines.orientate(getOrientation(), getX(), getY());
		}
	}

	/**
	 * detach or reattach unit to its machines
	 * @param detached new detached value
	 */
	public void setDetached(boolean detached) {
		this.detached = detached;
		if (detached) {
			projectilesNumber = 0;
			if (arrowSign != null) {
				arrowSign.setvalue(projectilesNumber);
				calculateWarriorsPositions();
			}
		}

	}
	/**
	 * teleport this unit to its destination
	 * 
	 */
	public void teleport() {
		super.teleport();
		if (!detached) {
			machines.teleport();
		}
	}
	/**
	 * get initial number of warriors as a string
	 * 
	 * @return initial number of warriors as a string
	 */
	public String getInitialNumberString() {
		return Integer.toString(
			getInitialNumber() / machines.getNormalCrewNumber())
			+ "&"
			+ Integer.toString(getInitialNumber());
	}
	/**
	 * get warriors number as a string
	 * 
	 * @return warriors number as a string
	 */
	public String getNumberString() {
		return Integer.toString(getNumber() / machines.getNormalCrewNumber())
			+ "&"
			+ Integer.toString(getNumber());
	}
	/**
	 * get warriors number as a string
	 * 
	 * @return warriors number as a string
	 */
	public String numberString() {
		return getNumberString();
	}



}
