/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.army.entities;

import java.util.ArrayList;
import java.util.List;

import fb.battlefieldelements.SignWithValue;
import fb.main.FB;
import fb.network.Client;
import fb.network.MessagesConstants;
import fb.util.Resources;
import fb.util.StringTools;

/**
 * an individual able to cast spells
 * 
 * @author farfpeuf
 * 
 */
public abstract class SpellCaster extends Individual {
	/**
	 * magic points
	 */
	private int magicPoints = 30, initialMagicPoints = 30;

	/**
	 * spell visual effects
	 */
	private List magics;

	/**
	 * additionnal sign to display remaining magic points
	 */
	private SignWithValue magicsign;

	// property change constants
	public static final String MAGIC_POINTS = "magicpoints";

	/**
	 * create inidividual from line read in catalog
	 * 
	 * @param ligne
	 *            catalog line
	 * @param linenumber
	 *            line number
	 */
	public SpellCaster(String ligne, int linenumber) {
		super(ligne, linenumber);
	}

	/**
	 * calculate individual cost
	 */
	public int getCost() {
		return super.getCost() + initialMagicPoints * 5;
	}

	/**
	 * initialize entity at the beginning of a battle
	 * 
	 */
	public void initBattle() {
		super.initBattle();
		magics = new ArrayList();
		magicsign = new SignWithValue(this, magicPoints);
		magicsign.setImage(Resources.getSign("magiccounter"));
		signs.add(1, magicsign);
	}

	/**
	 * reset entity after end of battle
	 * 
	 * @param winner
	 *            did the entity army won the battle
	 */
	public void endBattle(boolean winner) {
		super.endBattle(winner);
		magicsign = null;
		magics = null;
		magicPoints = initialMagicPoints;
	}

	/**
	 * restore lost magic points
	 * 
	 */
	public void restoreMagic() {
		if (isMoveAble() && (magicPoints < 1 + initialMagicPoints / 5)) {
			int bonus = 0;
			if (!suffersBadLight()) {
				bonus += 1;
			}
			if (getRemainingMove() == getMovement()) {
				bonus += 2;
			}
			if (bonus > 0) {
				if (magicPoints + bonus > (1 + initialMagicPoints / 5)) {
					bonus = (1 + initialMagicPoints / 5) - magicPoints;
				}
				magicPoints += bonus;
				String message;
				if (bonus == 1) {
					message = Resources.getString("restoremagic");
				} else {
					message = Resources.getString("restoremagics");
					message = StringTools.replaceAll(message, "<bonus>",
							Integer.toString(bonus));
				}

				message = StringTools.replaceAll(message, "<entity full name>",
						fullName);
				FB.getInstance().message(message, getColor());
				magicsign.setvalue(magicPoints);
			}
		}
	}

	/**
	 * consume magic points
	 * 
	 * @param mp
	 *            spent magic points
	 * @return false is too many points were spent
	 */
	public boolean spendMagicPoints(int mp) {
		if (FB.isNet()) {
			int oldmagicpoints = magicPoints;
			Client.getInstance().sendAndWait(
					MessagesConstants.SPENDMAGIC + getId() + "#" + mp);
			return (oldmagicpoints >= mp);
		} else {
			return doSpendMagicPoints(mp);
		}
	}

	/**
	 * do consume magic points
	 * 
	 * @param mp
	 *            spent magic points
	 * @return false is too many points were spent
	 */
	public boolean doSpendMagicPoints(int mp) {
		int oldValue = magicPoints;
		magicPoints -= mp;
		if (magicPoints < 0) {
			magicPoints = 0;
			magicsign.setvalue(magicPoints);
			firePropertyChange(MAGIC_POINTS, oldValue, magicPoints);
			return false;
		}
		firePropertyChange(MAGIC_POINTS, oldValue, magicPoints);
		magicsign.setvalue(magicPoints);
		return true;
	}

	/**
	 * is this individual casting a spell
	 * 
	 * @return true if individual is casting a spell
	 */
	public boolean isCastingSpell() {
		if (magics.size() > 0) {
			return true;
		}
		return model.getAction() == 'm';
	}

	/**
	 * start spell animation
	 * 
	 */
	public void animSpell() {
		model.setAction('m');
		model.setAnimationCounter(1);
		model.attribseq();
	}

	/**
	 * returns individual as a string to save it to a file
	 * 
	 * @return individual as a string
	 */
	public String saveString() {
		return super.saveString() + '\t' + magicPoints + '\t'
				+ initialMagicPoints;
	}

	/**
	 * load individual from a StringBuffer
	 * 
	 * @param buffy
	 *            the StringBuffer
	 */
	public void loadString(StringBuffer buffy) {
		super.loadString(buffy);
		magicPoints = StringTools.readBufferInt(buffy);
		initialMagicPoints = StringTools.readBufferInt(buffy);
	}

	/**
	 * bring individual back to life
	 * 
	 * @param priest
	 *            priest that resurected this individual
	 */
	public void resurection(Entity priest) {
		super.resurection(priest);
		magicPoints = 0;
		magicsign.setvalue(magicPoints);
	}

	/**
	 * @return the magics
	 */
	public List getMagics() {
		return magics;
	}

	/**
	 * @return Returns the initialMagicPoints.
	 */
	public int getInitialMagicPoints() {
		return initialMagicPoints;
	}

	/**
	 * @param initialMagicPoints
	 *            The initialMagicPoints to set.
	 */
	public void setInitialMagicPoints(int initialMagicPoints) {
		this.initialMagicPoints = initialMagicPoints;
	}

	/**
	 * @return Returns the magicpoints.
	 */
	public int getMagicPoints() {
		return magicPoints;
	}

	/**
	 * @param magicpoints
	 *            The magicpoints to set.
	 */
	public void setMagicPoints(int magicpoints) {
		this.magicPoints = magicpoints;
	}

}