/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.artificialintelligence;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import fb.army.entities.Banner;
import fb.army.entities.Courier;
import fb.army.entities.Entity;
import fb.army.entities.Herald;
import fb.army.entities.Hero;
import fb.army.entities.Heroic;
import fb.army.entities.Leader;
import fb.army.entities.Scout;
import fb.army.entities.Shootingunit;
import fb.army.entities.SpellCaster;
import fb.army.entities.Unit;
import fb.army.entities.Wizard;
import fb.army.hierarchy.AllyCommand;
import fb.army.hierarchy.Army;
import fb.army.hierarchy.Command;
import fb.army.hierarchy.Force;
import fb.army.hierarchy.MercenaryCommand;
import fb.army.hierarchy.StrangerCommand;
import fb.army.hierarchy.WarchiefCommand;
import fb.main.Catalog;
import fb.util.D6;
import fb.util.DX;
import fb.util.Resources;
import fb.util.StringTools;

/**
 * automatic army generator used by artificial intelligence in campaign game
 * 
 * @author farfpeuf
 * 
 */
public class ArmyRiser {

	/**
	 * create an army for the specified force the specified budget
	 * 
	 * @param force the force
	 * @param budget the maximum budget
	 * @return the army
	 */
	public static Army riseAI(Force force, int budget) {
		String race = Resources.getString(StringTools.replaceAll(force
				.getRace(), " ", ""));
		Army arm = new Army(race, null);
		int nbGroupsSup = budget / 800;
		int groupBudget = budget / (nbGroupsSup + 1);

		Command wchCommand = new WarchiefCommand();
		wchCommand.setName(Resources.getString("Warchiefcommand"));
		arm.recruitCommand(wchCommand);
		populateCommand(arm, wchCommand, groupBudget);
		budget -= wchCommand.getCost();

		int type;
		Command group = null;
		for (int i = 0; i < nbGroupsSup; i++) {
			type = DX.roll(3);
			if (type == 1 && !arm.hasMercenary()) {
				group = new MercenaryCommand();
				group.setName(Resources.getString("Mercenarycommand"));
			} else if (type == 2 && !arm.hasAllied()) {
				group = new AllyCommand();
				group.setName(Resources.getString("Alliedcommand"));
			} else {
				group = new Command();
				group.setName(Resources.getString("Othercommand"));
			}
			arm.recruitCommand(group);
			populateCommand(arm, group, groupBudget);
			budget -= group.getCost();
		}

		populateCommand(arm, wchCommand, budget);
		force.addArm(arm);
		force.setBudget(force.getBudget() - arm.getCost());
		return arm;
	}

	/**
	 * populate a command with entities with the specified budget
	 * @param arm the command army
	 * @param command the command
	 * @param budget the maximum budget to populate this group
	 */
	private static void populateCommand(Army arm, Command command, int budget) {

		List availableLeaders = getAvailableEntities((new Leader(null, 0))
				.getClass(), arm.getRace());
		int randomIndex = DX.roll(availableLeaders.size()) - 1;
		Leader leader = (Leader) availableLeaders.get(randomIndex);
		if (command instanceof MercenaryCommand
				|| command instanceof WarchiefCommand) {
			leader.setLeadership(5);
		} else {
			leader.setLeadership(4);
		}
		budget = riseEntity(leader, command, budget);

		String[] availableRaces = command.getAvailableRaces();
		String randomRace;
		Entity entityToRise;
		List availableEntities;

		while (budget > 50
				&& !(command instanceof StrangerCommand && arm.nbStrangers() + 1 > arm
						.nbNonStrangers())) {
			randomIndex = DX.roll(availableRaces.length) - 1;
			randomRace = availableRaces[randomIndex];

			if (command instanceof MercenaryCommand
					&& randomRace.equals(arm.getRace())) {
				randomRace = availableRaces[(randomIndex + 1)
						% availableRaces.length];
			}
			availableEntities = Catalog.getAvailableEntities(Resources
					.getString(randomRace.replaceAll(" ", "")));
			if (availableEntities == null) {
				System.out.println(Resources.getString(randomRace.replaceAll(
						" ", "")));
			} else {

				entityToRise = getEntityToRise(availableEntities, arm);

				if (entityToRise != null && entityToRise.isEnabled()) {
					if (entityToRise.isUnit()) {
						Unit unit = (Unit) entityToRise;
						if (budget > 100) {
							unit.setHasStandardBearer(true);
							if (unit.isShootingUnit()) {
								if (unit.isUpgradable()) {
									unit.upgradeLevel();
								}
							} else {
								unit.setHasChampion(true);
							}
							if (unit.isTerrible() || unit.getType() != 'c') {
								unit.setMusicians(1);
							}
						}

						int number = (2 * unit.getMax() + unit.getMin()) / 3;

						unit.setInitialNumber(number);
						while (number > unit.getMin() && unit.getCost() > budget) {
							number--;
							unit.setInitialNumber(number);
						}

					} else if (entityToRise instanceof SpellCaster) {

						SpellCaster spellCaster = (SpellCaster) entityToRise;
						int initialMp;
						if (budget > 100) {

							initialMp = 25 + DX.roll(10);
						} else {
							initialMp = (budget - spellCaster.getPrice()) / 5;
						}
						spellCaster.setInitialMagicPoints(initialMp);
						spellCaster.setMagicPoints(initialMp);
					}

					budget = riseEntity(entityToRise, command, budget);
				}
			}
		}

	}

	/**
	 * select an entity to add to the army
	 * @param availableEntities the list of all the available entities
	 * @param arm the army
	 * @return an entity
	 */
	private static Entity getEntityToRise(List availableEntities, Army arm) {
		int weight, maxWeight = 12;
		Entity potentialEntity, entityToRise = null;

		Iterator iter = availableEntities.iterator();

		while (iter.hasNext()) {

			potentialEntity = (Entity) iter.next();
			weight = getWeight(potentialEntity, arm);
			if (weight > maxWeight) {
				maxWeight = weight;
				entityToRise = potentialEntity;
			}
		}
		return entityToRise;
	}

	/**
	 * get the weight for an entity to be added to an army
	 * the entity with the maximum weight will be added
	 * @param entity the entity
	 * @param arm the army to which the enetity may be added
	 * @return the weight
	 */
	private static int getWeight(Entity entity, Army arm) {
		int weight = 0;
		for (int i = 0; i < 3; i++) {
			weight = weight + D6.roll();
		}
		if (entity.isUnit()) {
			int nbsu = arm.numberOf((new Shootingunit(null, 0)).getClass());
			int nbu = arm.numberOf((new Unit(null, 0)).getClass());
			if (entity.isShootingUnit()) {

				if (nbsu <= 1) {
					weight += 2;
				}
				if (nbsu == 0) {
					weight += 6;
				}
			} else {
				weight += 4;
				if (nbsu > 0 && nbsu == nbu) {
					weight += 4;
				}

			}

		} else if (entity.isLeader() || entity instanceof Banner
				|| entity instanceof Herald || entity instanceof Courier
				|| entity instanceof Scout) {
			weight -= Integer.MAX_VALUE;
		} else if (entity instanceof Heroic) {
			int nbhro = arm.numberOf((new Hero(null, 0)).getClass()
					.getSuperclass());
			if (nbhro == 0) {
				weight += 4;
			} else {
				weight -= Integer.MAX_VALUE;
			}
		} else if (entity instanceof SpellCaster) {
			int nbsp = arm.numberOf((new Wizard(null, 0)).getClass()
					.getSuperclass());
			if (nbsp == 0) {
				weight += 4;
			}
		}

		return weight;
	}

	/**
	 * adds the specified entity to the specified command if it is possible
	 * @param entity the entity to add
	 * @param command the command
	 * @param budget the old budget
	 * @return the new budget
	 */
	private static int riseEntity(Entity entity, Command command, int budget) {
		if (canRiseEntity(entity, command, budget)) {
			Entity clone = (Entity) entity.clone();
			command.addEntity(clone);
			budget -= entity.getCost();
		}
		return budget;
	}

	/**
	 * tests if an entity can be added to a command
	 * @param entity the entity
	 * @param command the command
	 * @param budget the available budget
	 * @return true if the entity can be added
	 */
	private static boolean canRiseEntity(Entity entity, Command command,
			int budget) {
		if (command instanceof StrangerCommand) {
			if (command.getArmy().nbStrangers() + entity.getNumberOfWarriors() > command
					.getArmy().nbNonStrangers()) {
				return false;
			}
		}
		return entity.getCost() <= budget;
	}

	/**
	 * list all the available entities of a specified type for a specified race
	 * @param type the wanted type
	 * @param race the wanted race 
	 * @return a list of entities
	 */
	private static List getAvailableEntities(Class type, String race) {
		List result = new ArrayList();
		Iterator iter = Catalog.getAvailableEntities(race).iterator();
		Entity entity;
		while (iter.hasNext()) {
			entity = (Entity) iter.next();
			if (type.isInstance(entity) && entity.isEnabled()) {
				result.add(entity);
			}
		}

		return result;
	}

}
