/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.artificialintelligence;

import java.awt.Point;
import java.awt.geom.AffineTransform;
import java.awt.geom.Area;

import fb.army.entities.Entity;
import fb.army.entities.Shootingunit;
import fb.main.FB;
import fb.util.GeometryTools;

/**
 * AI thread that finds the best next path step (considering obstacles) for an entity to
 * go to a new place or to go in contact with another entity
 * 
 * @author farfpeuf
 * 
 */
public class PathFinder extends Thread {

	private Entity entity, target;

	private int clickX, clickY, destX, destY;

	private boolean pathFound = false;

	private boolean stop = false;

	/**
	 * this thread is stopped after a determined number of milliseconds The
	 * best next path step found (if any) is then available with getDestX and getDestY
	 * methods
	 */
	public void safeStop() {
		stop = true;
		interrupt();
	}

	/**
	 * search for the next path step
	 */
	public void run() {
		int x, y, dx, dy, px, py, teta = 0, dex, dey, diamx = 0, diamy = 0, k = 0;
		boolean lommode = true;
		Area obstacles = entity.getMovementObstacles();
		if (target != null) {
			obstacles.subtract(target.getArea());
		}
		Point diamp, p;

		Point lomP0 = new Point();
		Point lomP1 = new Point();
		FB.setWaitCursor();
		x = entity.getX();
		y = entity.getY();
		dex = clickX - x;
		dey = clickY - y;
		for (int j = 0; j < 2 && !pathFound && !stop && !interrupted(); j++) {
			while (!pathFound && (k < 10 || (k < 30 && lommode))) {
				diamp = new Point(clickX - diamx, clickY - diamy);
				teta = 0;
				while (teta < 180 && !pathFound && !stop && !interrupted()) {
					p = new Point(diamp);
					GeometryTools.rotatePoint(teta, p, clickX, clickY);
					px = (int) p.getX();
					py = (int) p.getY();
					dx = px - x;
					dy = py - y;
					if (lommode) {
						lomP0.x = clickX;
						lomP0.y = clickY;
						lomP1.x = px;
						lomP1.y = py;
					}
					if (entity.canMoveTo(px, py, obstacles)
							&& (!lommode || !GeometryTools.hasIntersection(
									lomP0, lomP1, obstacles))) {
						Area testArea, contact = entity.getContactArea();
						destX = x;
						destY = y;
						int i = 1, X, Y, shortrange = 0, projectilesnumber = 0;
						boolean fin = false;
						if (entity instanceof Shootingunit) {
							shortrange = ((Shootingunit) entity)
									.getShortRange();
							projectilesnumber = ((Shootingunit) entity)
									.getProjectilesNumber();
						}
						// trouver le point dest
						while (!fin && !stop && !interrupted()) {

							if (i == 50) {
								X = dx;
								Y = dy;
							} else {
								X = (int) (((float) (dx * i)) / 50.0);
								Y = (int) (((float) (dy * i)) / 50.0);
							}

							testArea = new Area(AffineTransform
									.getTranslateInstance(X, Y)
									.createTransformedShape(contact));
							testArea.intersect(target.getArea());
							if (!testArea.isEmpty()
									|| (X * X + Y * Y) >= (entity
											.getRemainingMove()
											* 8 * entity.getRemainingMove() * 8)
									|| i == 50
									|| (entity.isEnnemy(target)
											&& (entity instanceof Shootingunit)
											&& (entity.translatedLineOfMovementTo(target,
													X, Y) < (shortrange * 8)) && projectilesnumber > 0)) {
								destX = x + X;
								destY = y + Y;
								fin = true;
							}
							i++;
						}
						synchronized (entity) {
							pathFound = true;
							entity.notify();
						}
					} // lomclear
					else {
						if (k == 0) {
							teta = 180;
						} else {
							if (teta >= 0) {
								teta += 10;
							}
							teta = -teta;
						}
					}
				} // while 2
				k++;
				if (lommode) {
					k++;
				}
				diamx = (int) (dex * ((double) k) / 10.0);
				diamy = (int) (dey * ((double) k) / 10.0);
			} // while 1
			lommode = false;
			diamx = 0;
			diamy = 0;
			k = 0;
		} // for
	}

	/**
	 * create a new pathfinder thread
	 * 
	 * @param entity
	 *            the entity for which a path is search
	 * @param target
	 *            the target entity or null if none
	 * @param clickX
	 *            the x coordinate to go to
	 * @param clickY
	 *            the y coordinate to go to
	 */
	public PathFinder(Entity entity, Entity target, int clickX, int clickY) {
		super();
		setName("PathFinder");
		this.entity = entity;
		this.target = target;
		this.clickX = clickX;
		this.clickY = clickY;
	}

	/**
	 * get the found destination x coordinate
	 * 
	 * @return destination x coordinate
	 */
	public int getDestX() {
		return destX;
	}

	/**
	 * get the found destination y coordinate
	 * 
	 * @return destination y coordinate
	 */
	public int getDestY() {
		return destY;
	}

	/**
	 * test if a path was found
	 * 
	 * @return true if a path was found
	 */
	public boolean isPathFound() {
		return pathFound;
	}
}
