/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.battlemapeditor;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.Point;
import java.awt.Polygon;
import java.awt.Rectangle;
import java.util.Enumeration;
import java.util.List;

import javax.swing.JPanel;

import fb.battlefield.MapModel;
import fb.battlefieldelements.VerticalSet;
import fb.util.FBColors;
import fb.util.FBImage;
import fb.util.Resources;

/**
 * main map editor panel containing map
 * 
 * @author farfpeuf
 * 
 */
public class MapPanel extends JPanel {
	/**
	 * the battle map model
	 */
	private MapModel mapModel;

	/**
	 * the map image
	 */
	private Image backgroundImage;

	/**
	 * constructor
	 * 
	 * @param mapModel
	 *            the map model
	 */
	public MapPanel(MapModel mapModel) {
		this.mapModel = mapModel;
		Dimension d = new Dimension(mapModel.getMapWidth(), mapModel
				.getMapLength());
		setSize(d);
		setPreferredSize(d);
		setMinimumSize(d);
		setBackground(FBColors.background);
		backgroundImage = Resources.getMap("", false, false);

		setOpaque(false);
	}

	/**
	 * change background image
	 * 
	 * @param name
	 *            image name
	 */
	public void setMapImage(String name) {
		backgroundImage = Resources.getMap(name, false, false);
		repaint();
	}

	/**
	 * paint a set of ploygons
	 * 
	 * @param type
	 *            the type of set
	 * @param g
	 *            graphics
	 */
	private void paintPolygonSet(char type, Graphics g) {
		List set = mapModel.getSet(type);
		if (set != null) {
			for (int i = 0; i < set.size(); i++) {
				g.drawPolygon((Polygon) set.get(i));
			}
		}
	}

	/**
	 * paint a set of rectangle
	 * 
	 * @param type
	 *            the type of set
	 * @param g2d
	 *            graphics
	 */
	private void paintRectangleSet(char type, Graphics2D g2d) {
		List set = mapModel.getSet(type);
		Rectangle rect;
		if (set != null) {
			for (int i = 0; i < set.size(); i++) {
				rect = (Rectangle) set.get(i);
				g2d.draw(rect);
				g2d.drawString(String.valueOf(type).toUpperCase(), (int) rect
						.getCenterX(), (int) rect.getCenterY());
			}
		}
	}

	/**
	 * paint the component
	 */
	public void paint(Graphics g) {
		g.setColor(Color.black);

		g.fillRect(0, 0, getWidth(), getHeight());

		g.drawImage(backgroundImage, 0, 0, mapModel.getMapWidth(), mapModel
				.getMapLength(), null);

		Point p, oldp;
		g.setColor(FBColors.primary);
		paintPolygonSet('r', g);
		g.setColor(FBColors.secondary);
		paintPolygonSet('w', g);
		Enumeration keys = mapModel.getVerticalSets().keys();
		String key;
		List vset;
		FBImage image;
		while (keys.hasMoreElements()) {
			key = (String) keys.nextElement();
			vset = (List) mapModel.getVerticalSets().get(key);
			image = VerticalSet.getFantasyImage(key);
			for (int i = 0; i < vset.size(); i++) {
				p = (Point) vset.get(i);
				image.draw(g, (int) p.getX(), (int) p.getY(), 1, 0, 0);
			}
		}
		g.setColor(Color.lightGray);
		paintPolygonSet('g', g);
		g.setColor(Color.black);
		paintPolygonSet('h', g);
		g.setColor(Color.cyan);
		paintPolygonSet('m', g);
		g.setColor(Color.orange);

		Graphics2D g2d = (Graphics2D) g;
		paintRectangleSet('a', g2d);
		g.setColor(FBColors.color2[0]);
		paintRectangleSet('b', g2d);
		g.setColor(FBColors.color2[1]);
		paintRectangleSet('c', g2d);
		g.setColor(FBColors.color2[2]);
		paintRectangleSet('d', g2d);
		g.setColor(FBColors.color2[3]);
		paintRectangleSet('e', g2d);
		g.setColor(FBColors.color2[4]);
		paintRectangleSet('f', g2d);
		g.setColor(FBColors.color2[5]);
		p = null;
		for (int i = 0; i < mapModel.getCurrentpoints().size(); i++) {
			p = (Point) mapModel.getCurrentpoints().get(i);
			if (Character.isLowerCase(mapModel.getCurrenttype())) {
				if (i == 0) {
					oldp = (Point) mapModel.getCurrentpoints().get(i);
				} else {
					oldp = p;
				}
				g.drawLine((int) p.getX(), (int) p.getY(), (int) oldp.getX(),
						(int) oldp.getY());
			} else {
				g.drawLine((int) p.getX() - 2, (int) p.getY() - 2, (int) p
						.getX() + 2, (int) p.getY() + 2);
				g.drawLine((int) p.getX() + 2, (int) p.getY() - 2, (int) p
						.getX() - 2, (int) p.getY() + 2);
			}
		}
		super.paint(g);
	}
}