/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.campaign;
import javax.swing.JPanel;

import fb.gui.panels.PlayersTable;
import fb.gui.widgets.ButtonListener;
import fb.gui.widgets.FBButton;
import fb.main.FB;
import fb.network.Client;
import fb.util.ArrayListModel;
import fb.util.FBColors;
import fb.util.PercentageLayout;
import fb.util.Resources;
/**
 * players panel for campaign games
 * @author Farfpeuf
 */
public class CampaignPlayersPanel extends JPanel {
	//singleton instance
	private static CampaignPlayersPanel instance;
	/**
	 * list of available races
	 */
	private static String[] racelist =
		new String[] {
			Resources.getString("aebarbarian"),
			Resources.getString("aegoblin"),
			Resources.getString("aedwarf"),
			Resources.getString("aewoodelf"),
			Resources.getString("aehuman"),
			Resources.getString("aeorc"),
			Resources.getString("aehigh-elf")};
	/**
	 * get singleton instance
	 * @return the singleton
	 */
	public static synchronized CampaignPlayersPanel getInstance() {
		if (null == instance) {
			instance = new CampaignPlayersPanel();
		}
		return instance;
	}

	//current number of players
	private int nbPlayers;
	//players table
	private PlayersTable plaTable;

	/**
	 * build and assemble panel components
	 *
	 */
	private CampaignPlayersPanel() {
		this.nbPlayers = 0;
		plaTable =
			new PlayersTable(true, Campaign.getInstance().getPlayersHandler().getLocalPlayersTableModel(),HexagonalMapModel.getInstance().getSides(),new ArrayListModel(racelist));
		setLayout(new PercentageLayout());
		setBackground(FBColors.background);
		FBButton addPlayer = new FBButton(Resources.getString("AddPlayer"));
		FBButton removePlayer = new FBButton(Resources.getString("RemovePlayer"));
		addPlayer.addActionListener(new ButtonListener() {
			public void threadedActionPerformed() {
				addPlayer();
			}
		});
		removePlayer.addActionListener(new ButtonListener() {
			public void threadedActionPerformed() {
				removePlayer();
			}
		});
		add(plaTable, new PercentageLayout.Constraint(0, 10, 100, 90));
		add(addPlayer, new PercentageLayout.Constraint(0, 0, 50, 10));
		add(removePlayer, new PercentageLayout.Constraint(50, 0, 50, 10));
		validate();
		repaint();
	}
	/**
	 * change number of players
	 * @param nbPlayers the new number of players
	 */
	public void setNbPlayer(int nbPlayers) {
		for (int i = this.nbPlayers; i < nbPlayers; i++) {
			addPlayer();
		}
	}
	/**
	 * add palyer to players list
	 *
	 */
	private void addPlayer() {
		if (nbPlayers < 6) {
			nbPlayers = nbPlayers + 1;
			String playerName = Resources.getString("player") + " " + (nbPlayers);
			if (FB.isNet()) {
				if (!Client.getInstance().getClientName().equals("")) {
					playerName =
						Client.getInstance().getClientName() + " " + playerName;
				}
			}

			String armyRace = racelist[((nbPlayers - 1) % racelist.length)];
			String side = null;
			ArrayListModel sides = HexagonalMapModel.getInstance().getSides();
			if (sides.isEmpty()) {
				sides.add("A");
				sides.add("B");
			}

			if (FB.isNet()) {
				side =
					(String) sides.get(
							Client.getInstance().getClientId() % sides.size());
			} else {
				side =
					(String) HexagonalMapModel.getInstance().getSides().get(
						(nbPlayers - 1) % sides.size());
			}

			plaTable.addPlayer(
				playerName,
				armyRace,
				false,
				side.charAt(0) - 'A');

		}
	}
	/**
	 * remove a player from players list
	 *
	 */
	private void removePlayer() {
		if (nbPlayers > 1) {
			nbPlayers = nbPlayers - 1;
			plaTable.removePlayer(nbPlayers);
		}
	}

}