/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.gui.frames;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;

import javax.swing.JPanel;

import fb.army.hierarchy.Army;
import fb.gui.panels.LocalPlayersPanel;
import fb.gui.panels.MapChooser;
import fb.gui.widgets.ButtonListener;
import fb.gui.widgets.FBButton;
import fb.gui.widgets.FBCheckBox;
import fb.main.FB;
import fb.main.Battle;
import fb.player.Player;
import fb.player.PlayersTableModel;
import fb.util.FileTools;
import fb.util.PercentageLayout;
import fb.util.Resources;

/**
 * frame used for local games
 * 
 * @author Farfpeuf
 */
public class LocalFrame extends JPanel {
	/**
	 * singleton instance
	 */
	private static LocalFrame instance;

	/**
	 * get singleton instance
	 * 
	 * @return the singleton
	 */
	public static synchronized  LocalFrame getInstance() {
		if (null == instance) {
			instance = new LocalFrame();
		}
		return instance;
	}

	/**
	 * delete singleton to free memory
	 * 
	 */
	public static synchronized  void freeInstance() {
		if (null != instance) {
			instance = null;
		}
	}

	/**
	 * constructor
	 * 
	 * build and assemble elements
	 * 
	 */
	public LocalFrame() {

		FBButton fight = new FBButton(Resources.getString("fightbutton"));
		FBButton back = new FBButton(Resources.getString("Exittomainmenu"));
		FBCheckBox autoDeployCheckBox = new FBCheckBox(Resources
				.getString("autodeploy"), Battle.getInstance().isAutoDeploy());
		autoDeployCheckBox.setToolTipText(Resources.getString("deployment.tt"));
		autoDeployCheckBox.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ev) {
				Battle.getInstance().setAutoDeploy(
						((FBCheckBox) ev.getSource()).getValue());
			}
		});

		if (FB.getInstance().isDebug()) {
			autoDeployCheckBox.setSelected(true);
		}

		// clic fight

		fight.addActionListener(new ButtonListener() {
			public void threadedActionPerformed() {

				if (testOnlyOneSide()) {
					FB.getInstance().popUpMessage(
							Resources.getString("onlyonesidetitle"),
							Resources.getString("onlyonesidetext"));
				} else {

					WaitFrame.setProgressionText(Resources
							.getString("readingarmylists")
							+ "\n");

					PlayersTableModel localPlas = Battle.getInstance()
							.getPlayersHandler().getLocalPlayersTableModel();
					Player pla;
					int ndPlayers = localPlas.getNbPlayer();
					Army[] armies = new Army[ndPlayers];
					for (int i = 0; i < ndPlayers; i++) {
						pla = localPlas.getPla(i);
						armies[i] = new Army(pla.getName(), FileTools
								.readFile("armies" + File.separator
										+ pla.getArmyName()), pla.isIa(), pla
								.getSide(), true);
						armies[i].setGmId(i);
					}

					FB.getInstance().playGame(armies, false, false);

				}
			}
		});

		back.addActionListener(new ButtonListener() {
			public void threadedActionPerformed() {
				FB.getInstance().displayMainMenuFrame();
			}
		});

		setLayout(new PercentageLayout());

		add(LocalPlayersPanel.getInstance(), new PercentageLayout.Constraint(25,
				10, 50, 30));
		add(MapChooser.getInstance(true), new PercentageLayout.Constraint(35, 43,
				30, 25));
		add(autoDeployCheckBox, new PercentageLayout.Constraint(39, 72, 22, 5));

		add(fight, new PercentageLayout.Constraint(15, 80, 15, 5));

		add(back, new PercentageLayout.Constraint(70, 80, 15, 5));

		setOpaque(false);
		validate();
		setBounds(0, 0, (int) (800 * FB.getXres()), (int) (600 * FB.getYres()));
	}

	/**
	 * test if all armies are on the same side
	 * 
	 * @return true if all armies aare on the same side
	 */
	public boolean testOnlyOneSide() {
		PlayersTableModel localPlas = Battle.getInstance().getPlayersHandler()
				.getLocalPlayersTableModel();
		Player pla;
		int ndPlayers = localPlas.getNbPlayer();
		int playerOneSide = localPlas.getPla(0).getSide();
		for (int i = 1; i < ndPlayers; i++) {
			pla = localPlas.getPla(i);
			if (pla.getSide() != playerOneSide) {
				return false;
			}
		}
		return true;
	}
}
