/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.gui.panels;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.BorderFactory;
import javax.swing.JLabel;
import javax.swing.border.Border;

import fb.army.entities.Entity;
import fb.army.entities.Leader;
import fb.army.entities.SpellCaster;
import fb.army.entities.Unit;
import fb.gui.widgets.EntityStateRenderer;
import fb.util.FBColors;
import fb.util.Resources;
/**
 * panel to display entity characteristics during a battle
 * @author farfpeuf
 *
 */
public class EntityBattlePanel
	extends EntityPanel
	implements PropertyChangeListener {
	/**
	 * additionnal cell to display entity morale
	 */
	private JLabel cellmorale;
	/**
	 * additionnal panel to display entity state
	 */
	private EntityStateRenderer entityStateRenderer;
	/**
	 * constructor
	 * build and assemble panel elements
	 */
	public EntityBattlePanel() {
		super();
		entityStateRenderer = new EntityStateRenderer(true);
		titlePanel.add(entityStateRenderer);
		setForeground(FBColors.background);
		cellmorale = new JLabel();
		rightPanel.add(cellmorale, 1);
		setSize(420, 100);
		validate();
	}
	/**
	 * display characteristics for specified entity
	 * @param e the entity
	 */
	public void display(Entity e) {
		if (this.entity != null) {
			if (this.entity instanceof SpellCaster) {
				this.entity.removePropertyChangeListener(
					SpellCaster.MAGIC_POINTS,
					this);
			}
			if (this.entity instanceof Unit) {
				this.entity.getCommand().removePropertyChangeListener(
					Unit.ORDER,
					this);
				this.entity.removePropertyChangeListener(Unit.NUMBER, this);
				this.entity.removePropertyChangeListener(Unit.MORALE, this);
				this.entity.removePropertyChangeListener(
					Unit.DISORGANIZED,
					this);
			}
			this.entity.removePropertyChangeListener(
				Entity.REMAINING_MOVE,
				this);
			this.entity.removePropertyChangeListener(Entity.STATE, this);
			this.entity.removePropertyChangeListener(Entity.LIFEPOINTS, this);
			this.entity.removePropertyChangeListener(Entity.SCALE, this);
		}
		super.setEntity(e);
		if (this.entity instanceof SpellCaster) {
			this.entity.addPropertyChangeListener(
				SpellCaster.MAGIC_POINTS,
				this);
		}
		if (e instanceof Unit) {
			this.entity.getCommand().addPropertyChangeListener(Unit.ORDER, this);
			this.entity.addPropertyChangeListener(Unit.NUMBER, this);
			this.entity.addPropertyChangeListener(Unit.MORALE, this);
			this.entity.addPropertyChangeListener(Unit.DISORGANIZED, this);
		}
		this.entity.addPropertyChangeListener(Entity.LIFEPOINTS, this);
		this.entity.addPropertyChangeListener(Entity.STATE, this);
		this.entity.addPropertyChangeListener(Entity.SCALE, this);
		this.entity.addPropertyChangeListener(Entity.REMAINING_MOVE, this);
		if (e.isUnit()) {
			cellmorale.setText(
				Resources.getString("Morale") + " : " + ((Unit) e).getMorale());
		} else {
			cellmorale.setText("");
		}
		cellmovement.setText(
			Resources.getString("Movement")
				+ " : "
				+ e.getRemainingMove()
				+ "/"
				+ e.getMovement());
		cellpdv.setText(Resources.getString("Pdv") + " : " + e.lifePointsString());
		Border border = BorderFactory.createLineBorder(e.getColor());
		photoPanel.setBorder(border);
		setTitle(e);
		entityStateRenderer.setEntity(e);

	}
	/**
	 * set panel title for specified entity
	 * @param e the entity
	 */
	private void setTitle(Entity e) {
		String name = e.getFullName();
		if (e.isUnit()) {
			name += " ("
				+ ((Unit) e).getNumberString()
				+ "/"
				+ ((Unit) e).getInitialNumberString()
				+ ")";
		}
		if (e.isLeader()) {
			Leader l = (Leader) e;
			name += " "
				+ (Resources.getString("Leadership") + " : " + l.getLeadership());
		}
		if (e.isMagic()) {
			SpellCaster m = (SpellCaster) e;
			name += " "
				+ (Resources.getString("Magic")
					+ " : "
					+ m.getMagicPoints()
					+ "/"
					+ m.getInitialMagicPoints());
		}
		title.setText(name);
		setToolTipText(name);
	}
	/**
	 * handle entity property change
	 */
	public synchronized void propertyChange(PropertyChangeEvent event) {
		if (event.getSource() instanceof Entity) {

			Entity source = (Entity) event.getSource();
			synchronized (source) {
				if (event.getPropertyName().equals(Entity.REMAINING_MOVE)) {
					cellmovement.setText(
						Resources.getString("Movement")
							+ " : "
							+ source.getRemainingMove()
							+ "/"
							+ source.getMovement());
				} else if (
					event.getPropertyName().equals(Entity.STATE)
						|| event.getPropertyName().equals(Unit.DISORGANIZED)) {
					entityStateRenderer.setEntity(source);

				} else if (
					event.getPropertyName().equals(SpellCaster.MAGIC_POINTS)) {
					setTitle(source);
				} else if (event.getPropertyName().equals(Unit.MORALE)) {
					cellmorale.setText(
						Resources.getString("Morale")
							+ " : "
							+ ((Unit) source).getMorale());
				} else if (event.getPropertyName().equals(Unit.NUMBER)) {
					String name = source.getFullName();
					name += " ("
						+ ((Unit) source).getNumberString()
						+ "/"
						+ ((Unit) source).getInitialNumberString()
						+ ")";
					title.setText(name);
				} else if (event.getPropertyName().equals(Entity.SCALE)) {
					cellstrength.setText(
						Resources.getString("Strength")
							+ " : "
							+ source.strengthString());
					cellresistance.setText(
						Resources.getString("Constitution")
							+ " : "
							+ source.resistanceString());
					cellworth.setText(
						Resources.getString("Attacks") + " : " + source.worthString());
				} else if (event.getPropertyName().equals(Entity.LIFEPOINTS)) {
					cellpdv.setText(
						Resources.getString("Pdv") + " : " + source.lifePointsString());
				}
			}
		}

	}
}