/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.gui.panels;

import java.awt.Color;
import java.awt.GridLayout;

import javax.swing.BorderFactory;
import javax.swing.JLabel;
import javax.swing.JPanel;

import fb.army.entities.Entity;
import fb.army.entities.Leader;
import fb.army.entities.SpellCaster;
import fb.army.entities.Unit;
import fb.gui.widgets.RecruitListener;
import fb.util.FBColors;
import fb.util.PercentageLayout;
import fb.util.Resources;

/**
 * panel to display entity characteristics and recruitment options
 * 
 * @author farfpeuf
 * 
 */
public class EntityRecruitPanel extends RecruitPanel {
	/**
	 * label to displays entity price
	 */
	private JLabel cellprice;

	/**
	 * panels for recruitment options
	 */
	private UnitRecruitPanel unitrecrut;

	private SpellCasterRecruitPanel magicrecrut;

	private LeaderRecruitPanel leaderrecrut;

	private JPanel currentrecrut = null;

	private JPanel allEntitiesPanel;

	/**
	 * current entity
	 */
	private Entity e;

	/**
	 * panel to display entity characteristics
	 * 
	 */
	private EntityPanel entityPanel;

	/**
	 * constructor build and assemble panel elements
	 */
	public EntityRecruitPanel() {

		super();

		allEntitiesPanel = new JPanel();
		entityPanel = new EntityPanel();
		allEntitiesPanel.setBorder(BorderFactory
				.createLineBorder(FBColors.lightBorder));
		allEntitiesPanel.setLayout(new GridLayout(1, 1));
		allEntitiesPanel.add(entityPanel);

		setBackground(Color.white);
		setForeground(FBColors.background);

		setLayout(new PercentageLayout());

		cellprice = new JLabel();

		unitrecrut = new UnitRecruitPanel();
		magicrecrut = new SpellCasterRecruitPanel();
		leaderrecrut = new LeaderRecruitPanel();

		unitrecrut.addRecrutListener(new RecruitListener() {
			public void entityChanged() {
				update();
			}
		});

		magicrecrut.addRecrutListener(new RecruitListener() {
			public void entityChanged() {
				update();
			}
		});

		leaderrecrut.addRecrutListener(new RecruitListener() {
			public void entityChanged() {
				update();
			}
		});

		entityPanel.rightPanel.add(cellprice, 1);

		add(unitrecrut, new PercentageLayout.Constraint(12, 60, 76, 30));

		add(leaderrecrut, new PercentageLayout.Constraint(34, 60, 32, 30));

		add(magicrecrut, new PercentageLayout.Constraint(34, 60, 32, 30));
		unitrecrut.setVisible(false);
		leaderrecrut.setVisible(false);
		magicrecrut.setVisible(false);

		add(allEntitiesPanel, new PercentageLayout.Constraint(0, 0, 100, 50));
		setSize(420, 185);
		setOpaque(false);
		validate();
		cellprice.setToolTipText(Resources.getString("price.tt"));

	}

	/**
	 * display characteristics and recruitment options for specified entity
	 * 
	 * @param e
	 *            the entity
	 */
	public void setEntity(Entity e) {
		entityPanel.setEntity(e);
		setToolTipText(e.getTooltip());
		this.e = e;
		if (currentrecrut != null) {
			currentrecrut.setVisible(false);
		}

		if (e.isUnit()) {
			unitrecrut.setUnit((Unit) e);
			currentrecrut = unitrecrut;

		} else if (e.isLeader()) {
			leaderrecrut.setLeader((Leader) e);
			currentrecrut = leaderrecrut;

		} else if (e.isMagic()) {
			magicrecrut.setSpellCaster((SpellCaster) e);
			currentrecrut = magicrecrut;

		} else {
			currentrecrut = null;
		}
		if (currentrecrut != null) {
			currentrecrut.setVisible(true);
		}

		if (currentrecrut != null) {
			currentrecrut.setBorder(allEntitiesPanel.getBorder());
		}
		entityPanel.cellmovement.setText(Resources.getString("Movement")
				+ " : " + e.movementString());
		entityPanel.cellpdv.setText(Resources.getString("Pdv") + " : "
				+ e.getInitialPdv());
		entityPanel.title.setText(e.getFullName());
		cellprice.setText(Resources.getString("Price") + " : "
				+ e.priceString());

	}

	/**
	 * update the panel
	 * 
	 */
	public void update() {
		entityPanel.celllevel.setText(Resources.getString("Level") + " : "
				+ e.levelString());
		fireEntityChanged();
	}

}
