/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.gui.panels;

import java.io.File;

import javax.swing.JPanel;

import fb.battlefield.MapModel;
import fb.gui.widgets.ButtonListener;
import fb.gui.widgets.FBButton;
import fb.main.FB;
import fb.main.Battle;
import fb.network.Client;
import fb.util.ArrayListModel;
import fb.util.FBColors;
import fb.util.PercentageLayout;
import fb.util.Resources;

/**
 * panel to manage this computer players
 * 
 * @author Farfpeuf
 */
public class LocalPlayersPanel extends JPanel {
	/**
	 * singleton instance
	 */
	private static LocalPlayersPanel instance;

	/**
	 * get singleton instance
	 * 
	 * @return the singleton
	 */
	public static synchronized LocalPlayersPanel getInstance() {
		if (null == instance) {
			instance = new LocalPlayersPanel();
		}
		return instance;
	}

	/**
	 * current number of players
	 */
	private int nbPlayers;

	/**
	 * the players table
	 */
	private PlayersTable plaTable = new PlayersTable(true, Battle.getInstance()
			.getPlayersHandler().getLocalPlayersTableModel());

	/**
	 * constructor build and assemble panel elements
	 */
	private LocalPlayersPanel() {
		this.nbPlayers = 0;
		setLayout(new PercentageLayout());
		setBackground(FBColors.background);
		FBButton addPlayer = new FBButton(Resources.getString("AddPlayer"));
		FBButton removePlayer = new FBButton(Resources
				.getString("RemovePlayer"));
		addPlayer.addActionListener(new ButtonListener() {
			public void threadedActionPerformed() {
				addPlayerAction();
			}
		});
		removePlayer.addActionListener(new ButtonListener() {
			public void threadedActionPerformed() {
				removePlayerAction();
			}
		});
		add(plaTable, new PercentageLayout.Constraint(0, 10, 100, 90));
		add(addPlayer, new PercentageLayout.Constraint(0, 0, 50, 10));
		add(removePlayer, new PercentageLayout.Constraint(50, 0, 50, 10));
		validate();
		repaint();
	}

	/**
	 * chnage number of players
	 * 
	 * @param nbPlayers
	 *            the new number of players
	 */
	public void setNbPlayer(int nbPlayers) {
		for (int i = this.nbPlayers; i < nbPlayers; i++) {
			addPlayerAction();
		}
	}

	/**
	 * add a player
	 * 
	 */
	private void addPlayerAction() {
		if (nbPlayers < 6) {
			nbPlayers = nbPlayers + 1;
			String playerName = Resources.getString("player") + " "
					+ (nbPlayers);
			if (FB.isNet()) {
				if (!Client.getInstance().getClientName().equals("")) {
					playerName = Client.getInstance().getClientName() + " "
							+ playerName;
				}
			}
			String[] filelist = (new File("armies")).list();
			String armyName = filelist[((nbPlayers - 1) % filelist.length)];
			String side = null;
			ArrayListModel sides = MapModel.getInstance().getSides();
			if (sides.isEmpty()) {
				sides.add("A");
				sides.add("B");
			}

			if (FB.isNet()) {
				side = (String) sides.get(Client.getInstance().getClientId()
						% sides.size());
			} else {
				side = (String) MapModel.getInstance().getSides().get(
						(nbPlayers - 1) % sides.size());
			}

			plaTable.addPlayer(playerName, armyName, false,
					side.charAt(0) - 'A');

		}
	}

	/**
	 * remove last player
	 * 
	 */
	private void removePlayerAction() {
		if (nbPlayers > 1) {
			nbPlayers = nbPlayers - 1;
			plaTable.removePlayer(nbPlayers);
		}
	}

}