/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.gui.panels;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.Stack;

import javax.swing.JScrollPane;
import javax.swing.ToolTipManager;
import javax.swing.tree.TreeNode;

import fb.army.hierarchy.HierarchyNode;
import fb.gui.widgets.armytreetable.ArmyNodeTreeCellRenderer;
import fb.gui.widgets.armytreetable.ArmyTreeModel;
import fb.gui.widgets.armytreetable.TreeTable;
import fb.gui.widgets.armytreetable.TreeTableModel;
import fb.gui.widgets.armytreetable.TreeTableModelAdapter;
import fb.util.FBColors;

/**
 * panel containing army tree table
 * 
 * @author farfpeuf
 * 
 */
public class TreeTablePanel extends JScrollPane implements
		PropertyChangeListener {

	/**
	 * the army tree table
	 */
	private TreeTable treeTable;

	/**
	 * constructor
	 * 
	 * @param root
	 *            root node
	 * @param battleMode
	 *            tree-table used in army editor or in battle information table ?
	 * 
	 */
	public TreeTablePanel(HierarchyNode root, boolean battleMode) {
		this(battleMode);
		setRoot(root);
	}

	/**
	 * set root node
	 * 
	 * @param root
	 *            the root node
	 */
	public void setRoot(HierarchyNode root) {
		TreeTableModelAdapter ad = (TreeTableModelAdapter) treeTable.getModel();
		((ArmyTreeModel) (ad.getTreeTableModel())).setRoot(root);

		if (root != null) {

			for (int i = 0; i < treeTable.getRowCount(); i++) {
				treeTable.getTree().expandRow(i);
			}

			// register changes to update the tree
			// @hint the stack is used to make a non recursive (faster) scan of
			// all the nodes
			Stack stack = new Stack();
			stack.push(root);
			HierarchyNode node;
			while (!stack.isEmpty()) {
				node = (HierarchyNode) stack.pop();

				node.addPropertyChangeListener(this);
				for (int i = 0; i < node.getChildCount(); i++) {
					stack.push(node.getChildAt(i));
				}

			}

		}
	}

	/**
	 * constructor
	 * 
	 * @param battleMode
	 *            tree-table used in army editor or in battle information table ?
	 * 
	 */
	public TreeTablePanel(boolean battleMode) {
		super();
		TreeTableModel armyTreeModel = new ArmyTreeModel(battleMode);

		treeTable = new TreeTable(armyTreeModel, battleMode);
		treeTable.getTree().setCellRenderer(
				new ArmyNodeTreeCellRenderer(battleMode));
		treeTable.setBackground(FBColors.background);
		ToolTipManager.sharedInstance().registerComponent(treeTable);
		setBackground(FBColors.background);
		setViewportView(treeTable);
		getViewport().setBackground(FBColors.background);
	}

	/**
	 * @return Returns the treeTable.
	 */
	public TreeTable getTreeTable() {
		return treeTable;
	}

	/**
	 * @return the army tree table model
	 */
	public TreeTableModelAdapter getArmyTableModel() {
		return (TreeTableModelAdapter) treeTable.getModel();
	}

	/**
	 * handle node changes
	 */
	public void propertyChange(PropertyChangeEvent evt) {
		TreeNode node = (TreeNode) evt.getSource();

		((ArmyTreeModel) getArmyTableModel().getTreeTableModel())
				.nodeChanged(node);
	}
}
