/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.gui.widgets;

import java.awt.Color;
import java.awt.event.AdjustmentEvent;
import java.awt.event.AdjustmentListener;

import javax.swing.JEditorPane;
import javax.swing.JScrollBar;
import javax.swing.JScrollPane;
import javax.swing.SwingUtilities;

import fb.main.FB;
import fb.util.FBColors;

/**
 * a fantasy battlefields custom text area
 * 
 * @author Farfpeuf
 */
public class FBTextArea extends JScrollPane {
	/**
	 * the area html text content
	 */
	private String htmlText = "";

	/**
	 * the area delegate pane
	 */
	private JEditorPane pane = new JEditorPane("text/html", "");

	/**
	 * must the text be scrolled down
	 */
	private boolean mustScrollDown = false;

	/**
	 * constructor
	 * 
	 */
	public FBTextArea() {
		this(true);
	}

	/**
	 * constructor
	 * 
	 * @param autoScroll
	 *            true if area must be scrolled down automatically when some
	 *            text is appended
	 */
	public FBTextArea(boolean autoScroll) {
		super();
		pane.setBackground(FBColors.background);
		pane.setForeground(FBColors.standardText);
		setViewportView(pane);
		if (autoScroll) {
			getVerticalScrollBar().addAdjustmentListener(
					new AdjustmentListener() {
						public void adjustmentValueChanged(
								final AdjustmentEvent e) {
							if (mustScrollDown) {
								mustScrollDown = false;
								SwingUtilities.invokeLater(new Runnable() {
									public void run() {
										JScrollBar scrollBar = (JScrollBar) e
												.getSource();
										int maximumValue = scrollBar
												.getMaximum()
												- scrollBar.getVisibleAmount();
										if (scrollBar.getValue() != maximumValue) {
											try {
												scrollBar
														.setValue(maximumValue);
											} catch (Exception e) {
												// ignore
											}
										}
									}
								});
							}
						}
					});
		}
	}

	/**
	 * append text at the end of the area
	 * 
	 * @param mess
	 *            the text to append
	 * @param color
	 *            the text color
	 */
	public void appendText(String mess, Color color) {
		String textColor = hexaString(color.getRed())
				+ hexaString(color.getGreen()) + hexaString(color.getBlue());
		htmlText += "<FONT COLOR=\"#" + textColor + "\">" + mess + "</FONT>"
				+ "<BR>";
		try {
			pane.setText(htmlText);
		} catch (Exception ignore) {
			FB.debug("exception in append text");
		}
		mustScrollDown = true;
	}

	/**
	 * covert rgb integer to html rgb string
	 * 
	 * @param value
	 *            the rgb integer
	 * @return the html rgb string
	 */
	private String hexaString(int value) {
		String result = Integer.toHexString(value);
		if (result.length() == 1) {
			result = "0" + result;
		}
		return result;
	}

	/**
	 * set the area text
	 * 
	 * @param mess
	 *            the text
	 * @param color
	 *            the text color
	 */
	public void setText(String mess, Color color) {
		String textColor = Integer.toHexString(color.getRed())
				+ Integer.toHexString(color.getGreen())
				+ Integer.toHexString(color.getBlue());
		htmlText = "<FONT COLOR=\"#" + textColor + "\">" + mess + "</FONT>"
				+ "<BR>";
		try {
			pane.setText(htmlText);
		} catch (Exception ignore) {

		}
		mustScrollDown = true;
	}

	/**
	 * set the text
	 * 
	 * @param text
	 *            the text
	 */
	public void setText(String text) {

		htmlText = text;
		try {
			pane.setText(htmlText);
		} catch (Exception ignore) {

		}
		mustScrollDown = true;
	}

	/**
	 * clear the text
	 * 
	 */
	public void clear() {
		htmlText = "";
		try {
			pane.setText(htmlText);
		} catch (Exception ignore) {

		}
	}

}