/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.gui.widgets.armytreetable;

import java.awt.Component;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;

import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTable;
import javax.swing.table.TableCellRenderer;

import fb.army.entities.Entity;
import fb.army.entities.Leader;
import fb.army.entities.SpellCaster;
import fb.army.entities.Unit;
import fb.army.hierarchy.Force;
import fb.gui.widgets.EntityStateRenderer;
import fb.util.FBColors;
import fb.util.Resources;

/**
 * renderer for army nodes special properties
 * @author Farfpeuf
 */
public class ArmySpecialRenderer extends JPanel implements TableCellRenderer {

	/**
	 * component used to render the entity state
	 */
	private EntityStateRenderer entityStateRenderer;
	/**
	 * renderer used in army editor or in battle information table ?
	 */
	private boolean battleMode;
	
	/**
	 * component used for painting
	 */
	private JLabel label;

	/**
	 * grid bag constraint used to assemble elements
	 */
	private static GridBagConstraints constraint = new GridBagConstraints(0, 0,
			1, 1, 0.0, 0.0, GridBagConstraints.WEST, GridBagConstraints.NONE,
			new Insets(0, 0, 0, 0), 0, 0);
/**
 * TableCellRenderer implementation
 */
	public Component getTableCellRendererComponent(JTable table, Object value,
			boolean isSelected, boolean hasFocus, int row, int column) {
		String stringValue = getNodeString(value);

		label.setText(stringValue);
		label.setBackground(FBColors.background);
		if (isSelected)
			label.setForeground(FBColors.highlightedText);
		else
			label.setForeground(FBColors.standardText);
		if (value instanceof Entity) {
			String tooltip = ((Entity) value).getTooltip();
			if (!tooltip.startsWith("<html>")) {
				tooltip = "<html>" + tooltip;
			} else {
				tooltip = tooltip.substring(0, tooltip.length() - 7);
			}
			if (battleMode) {
				Entity e = (Entity) value;
				if (e.getState() == 'a') {
					tooltip += Resources.getString("annihilatedimage.tt");
				} else if (e.getState() == 'r') {
					tooltip += Resources.getString("routedimage.tt");
				} else if (e.getState() == 's') {
					tooltip += Resources.getString("shakenimage.tt");
				} else if (e.getState() == 'b') {
					tooltip += Resources.getString("bloodlustimage.tt");
				}

			}
			if (value instanceof Unit) {
				Unit u = (Unit) value;

				if (battleMode) {
					char order = u.getOrder();
					if (order == 'a') {
						tooltip += Resources.getString("chargeimage.tt");
					} else if (order == 'o') {
						tooltip += Resources.getString("manoeuvreimage.tt");
					} else if (order == 'h') {
						tooltip += Resources.getString("defendimage.tt");
					}
				}

				if (battleMode && u.isDisorganized()) {
					tooltip += Resources.getString("disorganizedimage.tt");
				}
				if (u.hasChief()) {
					tooltip += Resources.getString("chiefimage.tt");
				}
				if (u.hasChampion()) {
					tooltip += Resources.getString("championimage.tt");
				}
				if (u.hasStandardBearer()) {
					tooltip += Resources.getString("standardimage.tt");
				}
				if (u.getMusicians() > 0) {
					tooltip += Resources.getString("musicianimage.tt");
				}
				if (u.isUpgraded()) {
					tooltip += Resources.getString("upgradedimage.tt");
				}

			}

			tooltip += "</html>";
			setToolTipText(tooltip);
			entityStateRenderer.setEntity((Entity) value);
			entityStateRenderer.setVisible(true);
		}

		else {
			setToolTipText(null);
			entityStateRenderer.setVisible(false);
		}
		return this;
	}
/**
 * convert node value to special string value
 * @param value the node
 * @return string to display in special column
 */
	private String getNodeString(Object value) {
		if (value instanceof Entity) {
			Entity e = (Entity) value;
			String options = "";
			if (e.isLeader()) {
				Leader l = (Leader) e;
				options += (Resources.getString("Leadership") + " : " + l
						.getLeadership());
			}
			if (e.isMagic()) {
				SpellCaster m = (SpellCaster) e;
				options += (Resources.getString("Magic") + " : " + m
						.getInitialMagicPoints());
			}
			return options;
		} else if(value instanceof Force){
			Force force=(Force)value;
			return Resources.getString("budget")+" : "+force.getBudget();
		}else{
			return null;
		}
	}
/**
 * constructor
 * @param battleMode renderer used in army editor or in battle information table ?
 */
	public ArmySpecialRenderer(boolean battleMode) {
		super();
		entityStateRenderer = new EntityStateRenderer(battleMode);
		this.battleMode = battleMode;
		label = new JLabel();
		setLayout(new GridBagLayout());
		constraint.gridx = 0;
		add(label, constraint);
		constraint.gridx++;
		add(entityStateRenderer);
		constraint.gridx++;

	}


}