/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.gui.widgets.armytreetable;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.event.MouseEvent;

import javax.swing.JTable;
import javax.swing.JTree;
import javax.swing.table.TableCellEditor;
import javax.swing.table.TableCellRenderer;
import javax.swing.tree.DefaultTreeSelectionModel;
import javax.swing.tree.MutableTreeNode;
import javax.swing.tree.TreeModel;
import javax.swing.tree.TreeSelectionModel;

import fb.gui.widgets.StringRenderer;
import fb.util.FBColors;

/**
 * a tree with columns
 * 
 * @author farfpeuf
 * 
 */
public class TreeTable extends JTable {
	/**
	 * the tree delegate
	 */
	protected TreeTableCellRenderer tree;

	/**
	 * constructor
	 * 
	 * @param treeTableModel
	 *            the data model
	 * @param battleMode
	 *            tree-table used in army editor or in battle information table ?
	 */
	public TreeTable(TreeTableModel treeTableModel, boolean battleMode) {
		super();

		setBackground(FBColors.background);
		tree = new TreeTableCellRenderer(treeTableModel);

		super.setModel(new TreeTableModelAdapter(treeTableModel, tree));

		TreeSelectionModel treeSelectionModel = new DefaultTreeSelectionModel() {
			{
				TreeTable.this.setSelectionModel(listSelectionModel);
			}
		};

		tree.setSelectionModel(treeSelectionModel);

		setRowHeight(20);
		tree.setRowHeight(20);
		setDefaultRenderer(TreeModel.class, tree);
		setDefaultRenderer(Integer.class, new StringRenderer());
		setDefaultRenderer(String.class, new StringRenderer());
		setDefaultRenderer(MutableTreeNode.class,
				new ArmySpecialRenderer(false));
		setDefaultEditor(TreeModel.class, new TreeTableCellEditor());
		setShowGrid(false);
		setIntercellSpacing(new Dimension(0, 0));
		setAutoCreateColumnsFromModel(false);

	}

	/**
	 * set the columns widths
	 * 
	 * @param widths
	 *            array of columns widths
	 */
	public void setColumnWidths(int[] widths) {
		for (int i = 0; i < widths.length; i++) {
			getColumnModel().getColumn(i).setPreferredWidth(widths[i]);
		}
	}

	/**
	 * @return the tree selection model
	 */
	public DefaultTreeSelectionModel getTreeSelectionModel() {
		return (DefaultTreeSelectionModel) tree.getSelectionModel();
	}

	/**
	 * renderer using jtree delegate to display the tree nodes.
	 */
	public class TreeTableCellRenderer extends JTree implements
			TableCellRenderer {
		/**
		 * the visible row
		 */
		protected int visibleRow;

		/**
		 * constructor
		 * 
		 * @param model
		 *            the tree data model
		 */
		public TreeTableCellRenderer(TreeModel model) {
			super(model);
		}

		/**
		 * Moves and resizes this component. Overidden to set height to table
		 * height
		 */
		public void setBounds(int x, int y, int w, int h) {
			super.setBounds(x, 0, w, TreeTable.this.getHeight());
		}

		/**
		 * paint the component Overidden to translate to row position
		 */
		public void paint(Graphics g) {
			g.translate(0, -visibleRow * getRowHeight());
			super.paint(g);
		}

		/**
		 * Returns the component used for drawing the cell. This method is used
		 * to configure the renderer appropriately before drawing.
		 * 
		 * @param table
		 *            the <code>JTable</code> that is asking the renderer to
		 *            draw; can be <code>null</code>
		 * @param value
		 *            the value of the cell to be rendered. It is up to the
		 *            specific renderer to interpret and draw the value. For
		 *            example, if <code>value</code> is the string "true", it
		 *            could be rendered as a string or it could be rendered as a
		 *            check box that is checked. <code>null</code> is a valid
		 *            value
		 * @param isSelected
		 *            true if the cell is to be rendered with the selection
		 *            highlighted; otherwise false
		 * @param hasFocus
		 *            if true, render cell appropriately. For example, put a
		 *            special border on the cell, if the cell can be edited,
		 *            render in the color used to indicate editing
		 * @param row
		 *            the row index of the cell being drawn. When drawing the
		 *            header, the value of <code>row</code> is -1
		 * @param column
		 *            the column index of the cell being drawn
		 */
		public Component getTableCellRendererComponent(JTable table,
				Object value, boolean isSelected, boolean hasFocus, int row,
				int column) {

			setBackground(table.getBackground());
			if (isSelected)
				setBackground(table.getForeground());
			else
				setForeground(table.getSelectionForeground());
			visibleRow = row;

			return this;
		}

	}

	/**
	 * get cell tooltip
	 */
	public String getToolTipText(MouseEvent event) {
		int hitRowIndex = rowAtPoint(event.getPoint());
		ArmyNodeTreeCellRenderer renderer = (ArmyNodeTreeCellRenderer) tree
				.getCellRenderer();
		Object value = getValueAt(hitRowIndex, 0);
		return renderer.getToolTip(value);
	}

	/**
	 * get current editing row
	 */
	public int getEditingRow() {
		return (getColumnClass(editingColumn) == TreeModel.class) ? -1
				: editingRow;
	}

	/**
	 * @return tree delegate
	 */
	public TreeTableCellRenderer getTree() {
		return tree;
	}

	/**
	 * Editor using jtree as delegate to interact with tree nodes.
	 */
	public class TreeTableCellEditor extends AbstractCellEditor implements
			TableCellEditor {
		public Component getTableCellEditorComponent(JTable table,
				Object value, boolean isSelected, int r, int c) {
			return tree;
		}
	}

}
