/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.main;

import java.awt.Color;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.DisplayMode;
import java.awt.GraphicsDevice;
import java.awt.GraphicsEnvironment;
import java.awt.Point;
import java.awt.Toolkit;
import java.io.File;
import java.util.Calendar;
import java.util.Date;
import java.util.Locale;
import java.util.ResourceBundle;

import javax.swing.BorderFactory;
import javax.swing.JDesktopPane;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.ToolTipManager;
import javax.swing.UIManager;

import fb.army.entities.Daemon;
import fb.army.entities.Entity;
import fb.army.entities.Wizard;
import fb.army.hierarchy.Army;
import fb.army.hierarchy.Force;
import fb.battlefield.MapModel;
import fb.battlefield.Sprite;
import fb.battlefieldelements.Projectile;
import fb.campaign.Campaign;
import fb.campaign.CampaignArmyEditor;
import fb.campaign.CampaignGameFrame;
import fb.campaign.CampaignMapEditor;
import fb.campaign.CampaignPlayersPanel;
import fb.campaign.CampaignPreparationFrame;
import fb.campaign.CampaignStatePanel;
import fb.campaign.HexagonalMapModel;
import fb.gui.frames.ArmyEditorFrame;
import fb.gui.frames.BattleFrame;
import fb.gui.frames.HostNetFrame;
import fb.gui.frames.JoinNetFrame;
import fb.gui.frames.LanFrame;
import fb.gui.frames.LocalFrame;
import fb.gui.frames.MainMenuFrame;
import fb.gui.frames.ManualFrame;
import fb.gui.frames.OptionFrame;
import fb.gui.frames.WaitFrame;
import fb.gui.frames.popups.FilePopup;
import fb.gui.frames.popups.MessagePopup;
import fb.gui.frames.popups.MutlipleFieldPopup;
import fb.gui.panels.ArmyEditorPanel;
import fb.gui.panels.BattleStatePanel;
import fb.gui.panels.ChatArea;
import fb.gui.panels.LocalPlayersPanel;
import fb.gui.panels.MapChooser;
import fb.gui.panels.MessageArea;
import fb.network.Client;
import fb.network.Server;
import fb.sound.MusicHandler;
import fb.sound.OggToWavGenerator;
import fb.sound.SoundPlayer;
import fb.util.DX;
import fb.util.FBColors;
import fb.util.FileArrayListModel;
import fb.util.FileTools;
import fb.util.GameInstaller;
import fb.util.NetworkDX;
import fb.util.Resources;
import fb.util.StringTools;

/**
 * fantasy battlefields main frame and main class
 * 
 * @author farfpeuf
 * 
 */
public class FB extends JFrame {

	/**
	 * game version
	 */
	private static final String VERSION = "0.8.6";

	/**
	 * singleton instance
	 */
	private static FB instance;

	/**
	 * batch options
	 */
	private boolean debug = false, batch = false, mini = false,
			graphicMode = true;

	/**
	 * frame background
	 */
	private JLabel background;

	/**
	 * screen resolution
	 */
	private static double xResolution, yResolution;

	/**
	 * frame content pane
	 */
	private JDesktopPane contentPane;

	/**
	 * is current cursor a waiting cursor ?
	 */
	private boolean waitCursorSet = false;

	/**
	 * cursors ids
	 */
	public static final int DEFAULT_CURSOR = 0, DEFAULT_WAIT_CURSOR = 1,
			TARGET_CURSOR = 2, NORTH_CURSOR = 3, EAST_CURSOR = 4,
			SOUTH_CURSOR = 5, WEST_CURSOR = 6, NORTH_EAST_CURSOR = 7,
			SOUTH_EAST_CURSOR = 8, SOUTH_WEST_CURSOR = 9,
			NORTH_WEST_CURSOR = 10, DRUM1 = 11, DRUM2 = 12, DRUM3 = 13,
			DRUM4 = 14, NO_CURSOR = 15;

	/**
	 * current battlfield cursor id
	 */
	private int battlefieldCursor;

	/**
	 * current cursor id
	 */
	private int currentCursor = -1;

	/**
	 * cursors array
	 */
	private Cursor[] cursors = new Cursor[16];

	/**
	 * current display mode
	 */
	private static DisplayMode oldMode;

	/**
	 * current graphics device
	 */
	private static GraphicsDevice graphicsDevice;

	public static void killNet() {
		Server.freeInstance();
		Client.freeInstance();
	}

	/**
	 * launch client in batch mode for text
	 * 
	 */
	public void batchClient() {
		try {
			batch = true;
			Thread.sleep(2000);
			Client.getInstance().setClientName("Client");

			if (Client.getInstance().connect("localhost", 10409)) {
				Client.getInstance().start();
			}
			Thread.sleep(500);

			Battle.getInstance().getPlayersHandler().getLocalPlayersTableModel().addPlayer(
					"client 1", "Dwarves-1", true, 3);
			/*
			 * Gm.getInstance().getPlayersHandler().getLocalPlas().addPla(
			 * "client 2", "Barbarians-1", true, 2);
			 */

		} catch (InterruptedException e) {
			e.printStackTrace();
		}

	}

	/**
	 * launch server in batch mode for text
	 * 
	 */
	public void batchServer() {
		try {
			batch = true;
			Server.getInstance().setSocketNumber(10409);
			Server.getInstance().start();
			Client.getInstance().setClientName("Serveur");

			if (Client.getInstance().connect("localhost", 10409)) {
				Client.getInstance().start();
			}
			Thread.sleep(500);
			Battle.getInstance().getPlayersHandler().getLocalPlayersTableModel().addPlayer(
					"server 1", "Men-1", true, 0);
			Battle.getInstance().getPlayersHandler().getLocalPlayersTableModel().addPlayer(
					"server 2", "Goblins-1", true, 1);

			Thread.sleep(10000);
			Server.getInstance().stopConnections();
		} catch (InterruptedException e) {
			e.printStackTrace();
		}

	}

	/**
	 * launch game in a mini frame (for tests at work ;)
	 * 
	 */
	public void mini() {
		mini = true;
		xResolution = 0.5;
		yResolution = 0.5;
		displayMainMenuFrame();
		setBounds(600, 600, 0, 0);
		setUndecorated(false);
		cursors[DEFAULT_CURSOR] = Cursor.getDefaultCursor();
		Options.setTooltipsEnabled(true);
		show();
	}

	/**
	 * play a game
	 * 
	 * @param armies
	 *            armies involved
	 * @param loaded
	 *            was this game loaded from a file
	 * @param campaign
	 *            is it a campaign game
	 */
	public void playGame(Army[] armies, boolean loaded, boolean campaign) {
		displayLoadingFrame();
		Battle.getInstance().setArmies(armies);
		Battle.getInstance().setCampaignGame(campaign);
		Battle.getInstance().setLoadedGame(false);
		if (campaign) {
			Battle.getInstance().setAutoDeploy(false);
		}
		WaitFrame.setProgressionValue(1);
		try {
			loadEverything(loaded);
			dispalyBattleFrame();
			Battle.getLoop().reset();
			Battle.getLoop().start();
		} catch (LoadingException e) {
			e.getCause().printStackTrace();
			handleLoadingException();
		}
	}

	/**
	 * programm main method
	 * 
	 * @param args
	 *            command line arguments
	 */
	public static void main(String[] args) {
		if (args.length > 0) {

			Options.setMusicOn(false);
			Options.setSoundOn(false);

			if (args[0].equalsIgnoreCase("s")) {
				instance = new FB();
				instance.batchServer();
			} else if (args[0].equalsIgnoreCase("c")) {
				instance = new FB();
				instance.batchClient();
			} else if (args[0].equalsIgnoreCase("m")) {
				instance = new FB();
				instance.mini();

			}

		} else {

			graphicsDevice = GraphicsEnvironment.getLocalGraphicsEnvironment()
					.getDefaultScreenDevice();
			oldMode = graphicsDevice.getDisplayMode();
			try {
				instance = new FB();
				instance.cursors[DEFAULT_CURSOR] = Toolkit.getDefaultToolkit()
						.createCustomCursor(
								(Resources.getCursor("default")).getImage(),
								new Point(0, 0), "sword");

				instance.setDefaultCursor();
				instance.setBattlefieldCursor(DEFAULT_CURSOR);
				Dimension screenDim = Toolkit.getDefaultToolkit()
						.getScreenSize();
				xResolution = ((double) screenDim.getWidth()) / 800.0;
				yResolution = ((double) screenDim.getHeight()) / 600.0;
				if (graphicsDevice.isFullScreenSupported()) {
					graphicsDevice.setFullScreenWindow(instance);
				} else {
					instance.setSize(screenDim);
					instance.setVisible(true);
				}
				DisplayMode mode = Options.getDisplayMode();
				/*
				 * if (mode == null) { mode = new DisplayMode(1024,768,16,75); }
				 */// TODO
				if (mode != null) {
					setDisplayMode(mode);
				}
				instance.displayMainMenuFrame();
			} catch (Exception e) {
				e.printStackTrace();
				safeExit();
			}
		}
	}

	/**
	 * change display mode
	 * 
	 * @param displayMode
	 *            new display mode
	 */
	public static void setDisplayMode(DisplayMode displayMode) {
		if (graphicsDevice.isFullScreenSupported()
				&& graphicsDevice.isDisplayChangeSupported()
				&& !displayMode.equals(graphicsDevice.getDisplayMode())) {
			graphicsDevice.setDisplayMode(displayMode);
		}
		Resources.free("fond1");
		Resources.free("fond3");
		xResolution = ((double) displayMode.getWidth()) / 800.0;
		yResolution = ((double) displayMode.getHeight()) / 600.0;
	}

	/**
	 * exit programm safely
	 * 
	 */
	public static void safeExit() {
		if (oldMode != null) {
			setDisplayMode(oldMode);
		}
		if (graphicsDevice != null && graphicsDevice.isFullScreenSupported()) {
			graphicsDevice.setFullScreenWindow(null);
		}
		System.exit(0);
	}

	/**
	 * main frame constructor initialize all the mess
	 */
	private FB() {

		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		setUndecorated(true);
		cursors[DEFAULT_WAIT_CURSOR] = Cursor
				.getPredefinedCursor(Cursor.WAIT_CURSOR);
		// options
		Options.getInstance();
		// installation du son
		File soundDirectoryPath = new File("wav");
		if (!soundDirectoryPath.exists()) {
			String oggPath = "ogg";
			if (FileTools.exists(oggPath)) {
				new GameInstaller();
				// Internationalisation
				Resources.setResourceBundle(ResourceBundle.getBundle(
						"Messages", new Locale(Options.getLanguage(), Options
								.getCountry())));

				new OggToWavGenerator();
			}
		}
		// Internationalisation
		Resources.setResourceBundle(ResourceBundle.getBundle("Messages",
				new Locale(Options.getLanguage(), Options.getCountry())));
		// tooltips
		ToolTipManager.sharedInstance().setInitialDelay(300);
		ToolTipManager.sharedInstance().setDismissDelay(10000);
		ToolTipManager.sharedInstance().setReshowDelay(200);
		ToolTipManager.sharedInstance().setEnabled(Options.isTooltipsEnabled());

		customizeUI();

		// son

		File mididir = new File("music");
		if (mididir.exists()) {
			String[] midilist = mididir.list();
			if (midilist != null) {
				MusicHandler.setTracks(midilist);
			}
		}

		if (Options.isMusicOn()) {
			MusicHandler.play(0);
		}
		setBackground(FBColors.background);
		// init container
		contentPane = new JDesktopPane();
		setContentPane(contentPane);
		contentPane.setBackground(FBColors.background);

	}

	/**
	 * join a lan game
	 * 
	 */
	public void joinLanAction() {
		MutlipleFieldPopup joinBox = new MutlipleFieldPopup("JoinALanGame",
				new String[] { "HostAddress", "ClientName" }, new String[] {
						"127.0.0.1",
						debug ? "Client" : System.getProperty("user.name",
								Resources.getString("ClientName")) });
		contentPane.add(joinBox, 0);
		String[] params = joinBox.getStrings();
		if (null != params) {
			displayJoinNetFrame();
			Client.getInstance().setClientName(params[1]);
			if (Client.getInstance().connect(params[0], 10409)) {
				Client.getInstance().start();
			}
		}
	}

	/**
	 * host a lan game
	 * 
	 */
	public void hostLanAction() {
		MutlipleFieldPopup hostBox = new MutlipleFieldPopup("HostALanGame",
				new String[] { "HostName" }, new String[] { debug ? "Server"
						: System.getProperty("user.name", Resources
								.getString("HostName")) });
		contentPane.add(hostBox, 0);
		String[] params = hostBox.getStrings();
		if (null != params) {
			Server.getInstance().setSocketNumber(10409);
			Server.getInstance().start();
			displayHostNetFrame();
			Client.getInstance().setClientName(params[0]);
			if (Client.getInstance().connect("localhost", 10409)) {
				Client.getInstance().start();
			}
		}
	}

	/**
	 * edit an army
	 * 
	 */
	public void editAction() {
		FilePopup editbox = new FilePopup("armies", "editarmy", "Choosearmy",
				false);
		contentPane.add(editbox, 0);
		String armyFile = editbox.getFile();
		if (null != armyFile) {
			displayArmyEditorFrame();
			ArmyEditorPanel.getInstance().openArmy(armyFile);
		}
	}

	/**
	 * open army editor
	 * 
	 */
	public void openEditor() {

		displayArmyEditorFrame();
		Army newArmy = new Army("barbarian", "armies" + File.separator
				+ "newArmy");
		ArmyEditorPanel.getInstance().setArmy(newArmy);

	}

	/**
	 * display main menu frame
	 * 
	 */
	public void displayMainMenuFrame() {
		setFrameCursor(DEFAULT_CURSOR);
		contentPane.removeAll();
		contentPane.repaint();
		background = new JLabel(Resources.getBackgroundImage("fond1"));
		MainMenuFrame.getInstance().enableButtons();
		background.add(MainMenuFrame.getInstance());
		background
				.setSize((int) (800 * xResolution), (int) (600 * yResolution));
		contentPane.add(background);
		validate();
		setSize((int) (800 * xResolution), (int) (600 * yResolution));
		repaint();
	}

	/**
	 * display network game frame
	 * 
	 */
	public void displayLanFrame() {
		setFrameCursor(DEFAULT_CURSOR);
		contentPane.removeAll();
		contentPane.repaint();
		background = new JLabel(Resources.getBackgroundImage("fond1"));
		LanFrame.getInstance().enableButtons();
		background.add(LanFrame.getInstance());
		background
				.setSize((int) (800 * xResolution), (int) (600 * yResolution));
		contentPane.add(background);
		validate();
		setSize((int) (800 * xResolution), (int) (600 * yResolution));
		repaint();
	}

	/**
	 * display game manual frame
	 */
	public void displayManualFrame() {
		contentPane.removeAll();
		contentPane.repaint();
		background = new JLabel(Resources.getBackgroundImage("fond3"));
		background.add(ManualFrame.getInstance());
		background
				.setSize((int) (800 * xResolution), (int) (600 * yResolution));
		contentPane.add(background);
		validate();
		setSize((int) (800 * xResolution), (int) (600 * yResolution));
		repaint();
	}

	/**
	 * diaplay game options frame
	 * 
	 */
	public void displayOptionFrame() {
		contentPane.removeAll();
		contentPane.repaint();
		background = new JLabel(Resources.getBackgroundImage("fond3"));
		background.add(OptionFrame.getInstance());
		background
				.setSize((int) (800 * xResolution), (int) (600 * yResolution));
		contentPane.add(background);
		validate();
		setSize((int) (800 * xResolution), (int) (600 * yResolution));
		repaint();
	}

	/**
	 * display local game frame
	 * 
	 */
	public void displayLocalFrame() {
		contentPane.removeAll();
		contentPane.repaint();
		background = new JLabel(Resources.getBackgroundImage("fond3"));
		// boutons etc
		background.add(LocalFrame.getInstance());
		LocalPlayersPanel.getInstance().setNbPlayer(2);
		background
				.setSize((int) (800 * xResolution), (int) (600 * yResolution));
		contentPane.add(background);
		setSize((int) (800 * xResolution), (int) (600 * yResolution));
		validate();
		repaint();
	}

	/**
	 * display campaign preparation frame
	 * 
	 */
	public void displayCampaignPreparationFrame() {
		contentPane.removeAll();
		contentPane.repaint();
		background = new JLabel(Resources.getBackgroundImage("fond3"));
		// boutons etc
		background.add(CampaignPreparationFrame.getInstance());
		CampaignPlayersPanel.getInstance().setNbPlayer(2);
		background
				.setSize((int) (800 * xResolution), (int) (600 * yResolution));
		contentPane.add(background);
		setSize((int) (800 * xResolution), (int) (600 * yResolution));
		validate();
		repaint();
	}

	/**
	 * display frame to join network game
	 * 
	 */
	public void displayJoinNetFrame() {
		contentPane.removeAll();
		contentPane.repaint();
		background = new JLabel(Resources.getBackgroundImage("fond3"));
		background.add(JoinNetFrame.getInstance());
		background
				.setSize((int) (800 * xResolution), (int) (600 * yResolution));
		contentPane.add(background);
		validate();
		repaint();
	}

	/**
	 * display frame to host a network game
	 * 
	 */
	public void displayHostNetFrame() {
		contentPane.removeAll();
		contentPane.repaint();
		background = new JLabel(Resources.getBackgroundImage("fond3"));
		background.add(HostNetFrame.getInstance());
		background
				.setSize((int) (800 * xResolution), (int) (600 * yResolution));
		contentPane.add(background);
		validate();
		repaint();
	}

	/**
	 * display army editor for campaign game
	 * 
	 * @param force
	 *            force for which an army is shall be risen
	 */
	public void displayCampaignArmyEditorFrame(Force force) {
		contentPane.removeAll();
		contentPane.repaint();
		background = new JLabel(Resources.getBackgroundImage("fond3"));
		background.add(new CampaignArmyEditor(force));
		background
				.setSize((int) (800 * xResolution), (int) (600 * yResolution));
		contentPane.add(background);
		validate();
		setSize((int) (800 * xResolution), (int) (600 * yResolution));
		repaint();
		String race = Resources.getString(StringTools.replaceAll(force
				.getRace(), " ", ""));

		Army newArmy = new Army(race, null);
		Point capital = HexagonalMapModel.getInstance().getCapital(
				force.getSide());
		if (capital != null) {
			newArmy.initPosition(capital);
		}
		ArmyEditorPanel.getInstance().setArmy(newArmy);
	}

	/**
	 * display army editor
	 * 
	 */
	public void displayArmyEditorFrame() {
		contentPane.removeAll();
		contentPane.repaint();
		background = new JLabel(Resources.getBackgroundImage("fond3"));
		background.add(new ArmyEditorFrame());
		background
				.setSize((int) (800 * xResolution), (int) (600 * yResolution));
		contentPane.add(background);
		validate();
		setSize((int) (800 * xResolution), (int) (600 * yResolution));
		repaint();
	}

	/**
	 * display waiting frame when loading a game
	 * 
	 */
	public void displayLoadingFrame() {
		contentPane.removeAll();
		contentPane.repaint();
		background = new JLabel(Resources.getBackgroundImage("fond3"));
		background.add(WaitFrame.getInstance());
		background
				.setSize((int) (800 * xResolution), (int) (600 * yResolution));
		contentPane.add(background);
		validate();
		setSize((int) (800 * xResolution), (int) (600 * yResolution));
		repaint();
	}

	/**
	 * clear the frame
	 * 
	 */
	public void clearFrame() {
		setFrameCursor(DEFAULT_WAIT_CURSOR);
		contentPane.removeAll();
		contentPane.repaint();
	}

	/**
	 * display campaign game frame
	 * 
	 */
	public void dispalyCampaignGameFrame() {
		background = new JLabel(Resources.getBackgroundImage("fond3"));
		background
				.setSize((int) (800 * xResolution), (int) (600 * yResolution));
		background.setLayout(null);
		CampaignGameFrame.getInstance().setBounds((int) (5 * xResolution),
				(int) (5 * yResolution), (int) (790 * xResolution),
				(int) (590 * yResolution));
		background.add(CampaignGameFrame.getInstance());
		contentPane.add(background);
		validate();
		setSize((int) (800 * xResolution), (int) (600 * yResolution));
		repaint();
		setDefaultCursor();
	}

	/**
	 * display campaign map editor
	 * 
	 */
	public void displayCampaignMapEditor() {
		contentPane.removeAll();
		contentPane.repaint();
		background = new JLabel(Resources.getBackgroundImage("fond3"));
		background
				.setSize((int) (800 * xResolution), (int) (600 * yResolution));
		background.setLayout(null);
		CampaignMapEditor.getInstance().setBounds((int) (5 * xResolution),
				(int) (5 * yResolution), (int) (790 * xResolution),
				(int) (590 * yResolution));
		background.add(CampaignMapEditor.getInstance());
		contentPane.add(background);
		validate();
		setSize((int) (800 * xResolution), (int) (600 * yResolution));
		repaint();
		CampaignMapEditor.getInstance().start();
	}

	/**
	 * display battle frame
	 * 
	 */
	public void dispalyBattleFrame() {
		contentPane.removeAll();
		contentPane.repaint();
		background = new JLabel(Resources.getBackgroundImage("fond3"));
		background
				.setSize((int) (800 * xResolution), (int) (600 * yResolution));
		background.setLayout(null);
		background.add(BattleFrame.getInstance());
		contentPane.add(background);
		validate();
		setSize((int) (800 * xResolution), (int) (600 * yResolution));
		repaint();
		SoundPlayer.getInstance().play("intro");
		BattleFrame.getInstance().start();

	}

	/**
	 * write a debug message
	 * 
	 * @param mess
	 *            the debug message
	 */
	public static void debug(String mess) {

		if (getInstance() != null && getInstance().debug) {
			try {
				Calendar cal = Calendar.getInstance();
				cal.setTime(new Date());
				String phaseName = (Battle.getLoop().getCurrentPhase() == null) ? "init"
						: Battle.getLoop().getCurrentPhase().getName();

				String threadName = Thread.currentThread().getName();
				String texte = "[" + cal.get(Calendar.MINUTE) + "."
						+ cal.get(Calendar.SECOND) + "."
						+ cal.get(Calendar.MILLISECOND) + " - " + phaseName
						+ " : " + Battle.getInstance().getTime() + "]" + " ("
						+ threadName + ") "
						+ Battle.getInstance().getActiveString() + " " + mess;
				System.out.println(texte);

			} catch (RuntimeException e) {
				e.printStackTrace();
			}
		}
	}

	/**
	 * handle unexpected exception
	 * 
	 * @param ex
	 *            the exception
	 */
	public static void exception(Exception ex) {
		if (getInstance() != null && getInstance().debug) {
			ex.printStackTrace();
			Toolkit.getDefaultToolkit().beep();
			debug("Exception : " + ex.getMessage());
		}
	}

	/**
	 * display a message
	 * 
	 * @param mess
	 *            the message
	 * @param color
	 *            the message color
	 */
	public void message(String mess, Color color) {
		if (batch) {
			debug(mess);
		} else {
			MessageArea.getInstance().message(mess, color);
		}
	}

	/**
	 * popup a message
	 * 
	 * @param title
	 *            popup tittle
	 * @param mess
	 *            the message
	 */
	public void popUpMessage(String title, String mess) {
		if (batch) {
			debug(mess);
		} else {
			MessagePopup messageBox = new MessagePopup(title, mess);
			messageBox.setBounds((int) (300 * FB.xResolution),
					(int) (270 * FB.yResolution), Math.max(
							(int) (200 * FB.xResolution), 80), Math.max(
							(int) (100 * FB.yResolution), 80));
			messageBox.setMinimumSize(messageBox.getPreferredSize());
			contentPane.add(messageBox);
			messageBox.toFront();
			messageBox.show();
		}
	}

	/**
	 * display a message
	 * 
	 * @param mess
	 *            the message
	 */
	public void message(String mess) {
		if (batch) {
			debug(mess);
		} else {
			MessageArea.getInstance().message(mess);
		}
	}

	/**
	 * @return about message
	 */
	public static String getAboutMessage() {
		StringBuffer result = new StringBuffer();
		result.append("Fantasy Battlefields");
		result.append(' ');
		result.append(Resources.getString("version"));
		result.append(' ');
		result.append(VERSION);
		result.append("\n\n");
		result.append(Resources.getString("musicAbout"));
		return result.toString();
	}

	/**
	 * display waiting message when loading
	 * 
	 */
	public void displayWaitingMessage() {
		WaitFrame.message(getAboutMessage() + "\n\n");
	}

	// --------------------------------------------------------------------------------------------------------------------------
	// loading
	// --------------------------------------------------------------------------------------------------------------------------
	/**
	 * load the sounds
	 */
	public void loadSounds() {
		if (Options.isSoundOn()
				&& (SoundPlayer.getInstance().progress() != 100)) {
			SoundPlayer.getInstance().loadall();
			while (SoundPlayer.getInstance().progress() < 100) {
				try {
					Thread.sleep(100);
					WaitFrame.setProgressionValue((int) (SoundPlayer
							.getInstance().progress() * 35 / 100));
				} catch (InterruptedException e) {
					System.out.println("interrupted");
				}
			}
		}
	}

	/**
	 * laod the cursors
	 * 
	 */
	public void loadCursors() {
		if (cursors[NO_CURSOR] == null) {

			cursors[NO_CURSOR] = Toolkit.getDefaultToolkit()
					.createCustomCursor((Resources.getCursor("no")).getImage(),
							new Point(0, 0), "no");

			cursors[TARGET_CURSOR] = Toolkit.getDefaultToolkit()
					.createCustomCursor(
							(Resources.getCursor("target")).getImage(),
							new Point(16, 16), "target");

			cursors[NORTH_CURSOR] = Toolkit.getDefaultToolkit()
					.createCustomCursor(
							(Resources.getCursor("narrow")).getImage(),
							new Point(16, 0), "north");

			cursors[EAST_CURSOR] = Toolkit.getDefaultToolkit()
					.createCustomCursor(
							(Resources.getCursor("earrow")).getImage(),
							new Point(31, 16), "east");

			cursors[SOUTH_CURSOR] = Toolkit.getDefaultToolkit()
					.createCustomCursor(
							(Resources.getCursor("sarrow")).getImage(),
							new Point(16, 31), "south");

			cursors[WEST_CURSOR] = Toolkit.getDefaultToolkit()
					.createCustomCursor(
							(Resources.getCursor("oarrow")).getImage(),
							new Point(0, 16), "west");

			cursors[NORTH_EAST_CURSOR] = Toolkit.getDefaultToolkit()
					.createCustomCursor(
							(Resources.getCursor("nearrow")).getImage(),
							new Point(31, 0), "n-e");

			cursors[SOUTH_EAST_CURSOR] = Toolkit.getDefaultToolkit()
					.createCustomCursor(
							(Resources.getCursor("searrow")).getImage(),
							new Point(31, 31), "s-e");

			cursors[SOUTH_WEST_CURSOR] = Toolkit.getDefaultToolkit()
					.createCustomCursor(
							(Resources.getCursor("soarrow")).getImage(),
							new Point(0, 31), "s-o");

			cursors[NORTH_WEST_CURSOR] = Toolkit.getDefaultToolkit()
					.createCustomCursor(
							(Resources.getCursor("noarrow")).getImage(),
							new Point(0, 0), "n-o");

			cursors[DRUM1] = Toolkit.getDefaultToolkit().createCustomCursor(
					(Resources.getCursor("drum1")).getImage(),
					new Point(16, 16), "DRUM1");

			cursors[DRUM2] = Toolkit.getDefaultToolkit().createCustomCursor(
					(Resources.getCursor("drum2")).getImage(),
					new Point(16, 16), "DRUM2");

			cursors[DRUM3] = Toolkit.getDefaultToolkit().createCustomCursor(
					(Resources.getCursor("drum3")).getImage(),
					new Point(16, 16), "DRUM3");

			cursors[DRUM4] = Toolkit.getDefaultToolkit().createCustomCursor(
					(Resources.getCursor("drum4")).getImage(),
					new Point(16, 16), "DRUM4");
		}
	}

	/**
	 * load the sprites
	 * 
	 */
	public void loadSprites() {
		Entity entity;
		boolean loadDaemon = false;
		for (int i = 0; i < Battle.getInstance().getActiveEntities().size(); i++) {
			entity = (Entity) Battle.getInstance().getActiveEntities().get(i);
			if (!loadDaemon && entity instanceof Wizard) {
				loadDaemon = true;
			}
			WaitFrame.setProgressionText(entity.getFullName());
			if (entity.getSprite() == null) {
				entity.createSprite();
			}
			WaitFrame.setProgressionValue(65 + (int) ((i + 1) * (30 / Battle
					.getInstance().getActiveEntities().size())));
		}
		if (loadDaemon) {
			((Entity) Catalog.getFullCatalog().get(Daemon.DAEMON_LINENUMBER))
					.createSprite();
		}
	}

	/**
	 * load every resources needed for the game
	 * 
	 * @param loadedGame
	 *            was game loaded from a file
	 * @throws LoadingException
	 */
	public void loadEverything(boolean loadedGame) throws LoadingException {
		loadEverything(loadedGame, null);
	}

	/**
	 * free memory before loading another game
	 * 
	 */
	private void freeMemoryBeforeLoading() {
		if (Options.isMusicOn()) {
			MusicHandler.stop();
		}
		graphicMode = !batch;

		if (graphicMode) {

			HostNetFrame.freeInstance();
			JoinNetFrame.freeInstance();
			LocalFrame.freeInstance();
			LanFrame.freeInstance();
			ManualFrame.freeInstance();
			OptionFrame.freeInstance();
			ArmyEditorPanel.freeInstance();
			MainMenuFrame.freeInstance();
			CampaignPreparationFrame.freeInstance();
			CampaignGameFrame.freeInstance();
		}

		System.gc();
	}

	/**
	 * load every resources needed for the game
	 * 
	 * @param loadedGame
	 *            was game loaded from a file
	 * @param mapName
	 *            game map name
	 * @throws LoadingException
	 */
	public void loadEverything(boolean loadedGame, String mapName)
			throws LoadingException {
		freeMemoryBeforeLoading();
		if (!loadedGame) {
			if (isNet()) {
				Battle.getInstance().setTime(NetworkDX.roll(10));
			} else {
				Battle.getInstance().setTime(DX.roll(10));
			}
		}

		if (graphicMode) {
			displayWaitingMessage();
			WaitFrame.setProgressionText(Resources.getString("loadingsounds")
					+ "\n");
			loadSounds();

			WaitFrame.setProgressionValue(35);
			WaitFrame.setProgressionText(Resources.getString("loadingcursors")
					+ "\n");
			loadCursors();

			WaitFrame.setProgressionValue(40);
			WaitFrame.setProgressionText(Resources.getString("loadingphases")
					+ "\n");
			Resources.loadPhasesIcons();

			WaitFrame.setProgressionValue(45);
			WaitFrame.setProgressionText(Resources.getString("loadingarrows")
					+ "\n");
			Projectile.loadProjectilesIcons();

			WaitFrame.setProgressionValue(50);
			WaitFrame.setProgressionText(Resources.getString("loadingtime")
					+ "\n");
			Resources.loadTimeIcons();

			WaitFrame.setProgressionValue(60);
			WaitFrame.setProgressionText(Resources.getString("creatingarmies")
					+ "\n");

		}
		Battle.getInstance().loadArmies();
		if (graphicMode) {
			WaitFrame.setProgressionValue(65);
			WaitFrame.setProgressionText(Resources.getString("loadingsprites")
					+ "\n");
			loadSprites();
			WaitFrame.setProgressionValue(95);
			WaitFrame.setProgressionText(Resources
					.getString("initializingmodels")
					+ "\n");
		}
		BattleFrame.getInstance().buildActionPanels();
		Battle.getInstance().initializeModels();
		if (graphicMode) {
			WaitFrame.setProgressionValue(97);
			WaitFrame.setProgressionText(Resources
					.getString("initializingobstacles")
					+ "\n");
		}
		if (graphicMode) {
			if (mapName == null) {
				Battle.getInstance().setCurrentMap(
						MapChooser.getInstance().getSelectedMap());
			} else {
				Battle.getInstance().setCurrentMap(mapName);
			}
		} else {
			Battle.getInstance().setCurrentMap("forest");
		}
		if (graphicMode) {
			BattleStatePanel.getInstance().buildTabs();
		}
		MapModel.getInstance().load(

		"mapmodels" + File.separator + Battle.getInstance().getCurrentMap());
		if (graphicMode) {
			BattleFrame.getInstance().init(Battle.getInstance().getCurrentMap(),
					MapModel.getInstance().getMapWidth(),
					MapModel.getInstance().getMapLength(),
					MapModel.getInstance().isFlipLR(),
					MapModel.getInstance().isFlipUD());
			BattleFrame.getInstance().displayEntityCharacteristics(
					(Entity) Battle.getInstance().getActiveEntities().get(0));

		}
		Battle.getInstance().initializeObstacles();
		if (graphicMode) {
			WaitFrame.freeInstance();
			Resources.free("fond1");
			Resources.free("fond3");
			Resources.free();
			Sprite.free();
		}
		System.gc();
		if (graphicMode) {
			WaitFrame.setProgressionValue(100);
			if (Options.isMusicOn()) {
				MusicHandler.playRandomTrack();
			}
		}
	}

	/**
	 * set waiting cursor
	 * 
	 */
	public static void setWaitCursor() {
		instance.waitCursorSet = true;
		instance.battlefieldCursor = -1;
	}

	/**
	 * change battlefield cursor
	 * 
	 * @param c
	 *            the new cursor index
	 */
	public void setBattlefieldCursor(int c) {
		if (battlefieldCursor != c) {
			waitCursorSet = false;
			battlefieldCursor = c;
		}
	}

	/**
	 * set default cursor
	 * 
	 */
	public void setDefaultCursor() {
		setFrameCursor(DEFAULT_CURSOR);
	}

	/**
	 * change cursor
	 * 
	 * @param c
	 *            the new cursor index
	 */
	public void setFrameCursor(int c) {
		if (currentCursor != c && !mini) {
			if (cursors[c] != null) {
				currentCursor = c;
				setCursor(cursors[c]);
			} else {
				setDefaultCursor();
			}
		}
	}

	/**
	 * WARNING, this is NOT a singleton get frame instance
	 * 
	 * @return the frame instance
	 */
	public static FB getInstance() {
		return instance;
	}

	/**
	 * save current game to file
	 * 
	 */
	public void saveGame() {
		FilePopup saveAsBox = new FilePopup("games", "Saveas", "Saveas", true);
		saveAsBox.setBounds((int) (300 * FB.xResolution),
				(int) (500 * FB.yResolution), (int) (200 * FB.xResolution),
				(int) (70 * FB.yResolution));

		saveAsBox.setMinimumSize(saveAsBox.getPreferredSize());
		FB.getInstance().getContentPane().add(saveAsBox);

		String gameFile = saveAsBox.getFile();
		if (null != gameFile) {
			saveGame(gameFile);
		}
	}

	/**
	 * campaign automatic save
	 * 
	 */
	public void autoSaveCampaign() {
		String gameFile = "games" + File.separator + "automatic_save";
		saveCampaign(false, gameFile);
	}

	/**
	 * battle automatic save
	 * 
	 */
	public void autoSaveGame(boolean campaign) {
		String gameFile = "games" + File.separator + "automatic_save";
		if (campaign) {
			saveCampaign(true, gameFile);
		} else {
			saveGame(gameFile);
		}
	}

	/**
	 * save current game to file
	 * 
	 * @param gameFile
	 *            the file
	 */
	private void saveGame(String gameFile) {
		if (null != gameFile) {
			FileTools.writeFile(gameFile, "n\t"
					+ Battle.getInstance().saveBattleString());
			message(Resources.getString("gameSaved"));
			FileArrayListModel.refresh("games");
		}
	}

	/**
	 * save current campaign to file
	 * 
	 * @param saveGame
	 *            also save game ?
	 */
	public void saveCampaign(boolean saveGame) {
		FilePopup saveAsBox = new FilePopup("games", "Saveas", "Saveas", true);
		saveAsBox.setBounds((int) (300 * FB.xResolution),
				(int) (500 * FB.yResolution), Math.max(80,
						(int) (200 * FB.xResolution)), Math.max(80,
						(int) (70 * FB.yResolution)));

		saveAsBox.setMinimumSize(saveAsBox.getPreferredSize());
		FB.getInstance().getContentPane().add(saveAsBox);

		String gameFile = saveAsBox.getFile();
		if (null != gameFile) {
			saveCampaign(saveGame, gameFile);
		}
	}

	/**
	 * save current campaign to file
	 * 
	 * @param saveGame
	 *            also save game ?
	 * @param gameFile
	 *            game file
	 */
	private void saveCampaign(boolean saveGame, String gameFile) {

		FileTools.writeFile(gameFile, "y\t"
				+ Campaign.getInstance().saveString(saveGame));
		message(Resources.getString("gameSaved"));
		FileArrayListModel.refresh("games");
	}

	/**
	 * load a game
	 * 
	 */
	public void loadGameAction() {
		FilePopup loadBox = new FilePopup("games", "loadGame", "Choosegame",
				false);
		contentPane.add(loadBox, 0);
		String gameFile = loadBox.getFile();
		if (gameFile != null) {
			loadGame(gameFile);
		}
	}

	/**
	 * load a game from a file
	 * 
	 * @param game
	 *            file content
	 */
	public void loadGame(String game) {
		FB.getInstance().clearFrame();

		try {
			StringBuffer buffy = new StringBuffer(FileTools.readFile(game));
			boolean isCampaign = (StringTools.readBufferChar(buffy) == 'y') ? true
					: false;
			if (isCampaign) {

				String[] cmpGmTokens = buffy.toString()
						.split(Campaign.GAME_TAG);

				Campaign.getInstance().loadString(cmpGmTokens[0]);

				CampaignStatePanel.getInstance().buildTabs();

				if (cmpGmTokens.length > 1) {
					displayLoadingFrame();
					WaitFrame.setProgressionText(Resources
							.getString("readingarmylists")
							+ "\n");
					WaitFrame.setProgressionValue(1);
					buffy = new StringBuffer(cmpGmTokens[1]);
					Battle.getInstance().loadCampaignBatlePart1(buffy);
					FB.getInstance().loadEverything(true,
							Battle.getInstance().getCurrentMap());
					Battle.getInstance().loadPart2(buffy);

					Battle.getInstance().setCampaignGame(true);
					Battle.getInstance().setLoadedGame(true);

					dispalyBattleFrame();
					Battle.getLoop().start();
				} else {
					dispalyCampaignGameFrame();
					CampaignGameFrame.getInstance().start();
				}
			} else {
				displayLoadingFrame();
				WaitFrame.setProgressionText(Resources
						.getString("readingarmylists")
						+ "\n");
				WaitFrame.setProgressionValue(1);
				Battle.getInstance().loadBattlePart1(buffy);
				loadEverything(true, Battle.getInstance().getCurrentMap());
				Battle.getInstance().loadPart2(buffy);

				Battle.getInstance().setCampaignGame(false);
				Battle.getInstance().setLoadedGame(true);

				dispalyBattleFrame();
				Battle.getLoop().start();
			}
		} catch (LoadingException e) {
			e.getCause().printStackTrace();
			handleLoadingException();
		}
		setDefaultCursor();
	}

	/**
	 * @return xResolution
	 */
	public static double getXres() {
		return xResolution;
	}

	/**
	 * @return yResolution
	 */
	public static double getYres() {
		return yResolution;
	}

	/**
	 * add a message to chat area
	 * 
	 * @param clientId
	 *            chat client id
	 * @param toAll
	 *            is message destinated to every players ?
	 * @param text
	 *            message
	 */
	public static void chat(int clientId, boolean toAll, String text) {
		ChatArea.getInstance().writeText(clientId, text,
				FBColors.color[clientId]);
	}

	/**
	 * center battlefield view on an army node
	 * 
	 * @param node
	 *            the army node
	 */
	public void centerOn(Object node) {
		BattleFrame.getInstance().centerOn(node);
	}

	/**
	 * customize swing UI
	 * 
	 */
	private static void customizeUI() {
		UIManager.put("ToolTip.background", FBColors.darkIvory);
		UIManager.put("ToolTip.border", BorderFactory
				.createLineBorder(FBColors.background));
		UIManager.put("ComboBox.selectionBackground", FBColors.background);
		UIManager.put("ComboBox.selectionForeground", FBColors.highlightedText);
		UIManager.put("ComboBox.background", FBColors.background);
		UIManager.put("ComboBox.foreground", FBColors.standardText);
		UIManager.put("ComboBox.buttonBackground", FBColors.background);
		UIManager.put("ComboBox.buttonShadow", FBColors.background);
		UIManager.put("ComboBox.buttonDarkShadow", FBColors.primary);
		UIManager.put("ComboBox.buttonHighlight", FBColors.background);
		UIManager.put("List.selectionBackground", FBColors.background);
		UIManager.put("List.selectionForeground", FBColors.highlightedText);
		UIManager.put("List.background", FBColors.background);
		UIManager.put("List.foreground", FBColors.standardText);
		UIManager.put("Panel.background", FBColors.background);
		UIManager.put("Label.foreground", FBColors.standardText);
		UIManager.put("Label.background", FBColors.background);
		UIManager.put("PopupMenu.background", FBColors.background);
		UIManager.put("MenuItem.background", FBColors.background);
		UIManager.put("MenuItem.foreground", FBColors.standardText);
		UIManager.put("MenuItem.selectionForeground", FBColors.highlightedText);
		UIManager.put("MenuItem.selectionBackground", FBColors.background);
		UIManager.put("PopupMenu.border", BorderFactory
				.createLineBorder(FBColors.primary));
		UIManager.put("MenuItem.border", BorderFactory.createEmptyBorder());
		UIManager.put("ScrollPane.border", BorderFactory.createEmptyBorder());
		UIManager.put("TableHeader.background", FBColors.background);
		UIManager.put("TableHeader.foreground", FBColors.standardText);
		UIManager.put("TableHeader.cellBorder", BorderFactory
				.createLineBorder(FBColors.highlightedText));
		UIManager.put("Table.gridColor", FBColors.primary);
		UIManager.put("TabbedPane.background", FBColors.background);
		UIManager.put("TabbedPane.foreground", FBColors.standardText);
		UIManager.put("TabbedPane.selected", FBColors.primary);
		UIManager.put("ScrollBar.background", FBColors.background);
		UIManager.put("ScrollBar.trackShadow", FBColors.background);
		UIManager.put("ScrollBar.trackHighlight", FBColors.background);
		UIManager.put("ScrollBar.thumbShadow", FBColors.background);
		UIManager.put("ScrollBar.thumbHighlight", FBColors.primary);
		UIManager.put("ScrollBar.thumb", FBColors.background);
		UIManager.put("Slider.background", FBColors.background);
		UIManager.put("Slider.shadow", FBColors.background);
	}

	/**
	 * @return Returns the batch.
	 */
	public boolean isBatch() {
		return batch;
	}

	/**
	 * @return Returns the debug.
	 */
	public boolean isDebug() {
		return debug;
	}

	/**
	 * @return Returns the graphicMode.
	 */
	public boolean isGraphicMode() {
		return graphicMode;
	}

	/**
	 * @return Returns the waitCursorSet.
	 */
	public boolean isWaitCursorSet() {
		return waitCursorSet;
	}

	/**
	 * @return true if current game is a network game
	 */
	public static boolean isNet() {
		return Client.hasInstance();
	}

	/**
	 * @return Returns the graphicsDevice.
	 */
	public static GraphicsDevice getGraphicsDevice() {
		return graphicsDevice;
	}

	/**
	 * @return Returns the battlefieldCursor.
	 */
	public int getBattlefieldCursor() {
		return battlefieldCursor;
	}

	/**
	 * @return Returns the currentCursor.
	 */
	public int getCurrentCursor() {
		return currentCursor;
	}

	/**
	 * @return Returns the mini.
	 */
	public boolean isMini() {
		return mini;
	}

	/**
	 * handle a file loading exception
	 * 
	 */
	public void handleLoadingException() {
		displayMainMenuFrame();
		popUpMessage(Resources.getString("loadingError"), Resources
				.getString("IncompatibleFile"));
	}

}