/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.main;

import java.awt.DisplayMode;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Properties;
import java.util.StringTokenizer;

/**
 * game options
 * 
 * @author farfpeuf
 * 
 */
public class Options extends Properties {

	/**
	 * singleton instance
	 */
	private static Options instance;

	/**
	 * get singleton instance
	 * 
	 * @return the singleton
	 */
	public static synchronized Options getInstance() {
		if (null == instance) {
			instance = new Options();
		}
		return instance;
	}

	/**
	 * is sound on?
	 */
	private boolean soundOn;

	/**
	 * is music on ?
	 */
	private boolean musicOn;

	/**
	 * are tooltips enabled ?
	 */
	private boolean tooltipsEnabled;

	/**
	 * current language
	 */
	private String language;

	/**
	 * current country
	 */
	private String country;

	/**
	 * current display mode
	 */
	private DisplayMode displayMode;

	/**
	 * constructor load options from file
	 */
	private Options() {
		load();
	}

	/**
	 * load options from properties file
	 * 
	 */
	public void load() {
		FileInputStream inputStream;

		try {
			inputStream = new FileInputStream("fb.properties");
			load(inputStream);
			soundOn = Boolean.valueOf(getProperty("sound", "true"))
					.booleanValue();
			musicOn = Boolean.valueOf(getProperty("music", "true"))
					.booleanValue();
			language = getProperty("language", "en");
			country = getProperty("country", "UK");
			String diplayModeString = getProperty("display");
			if (diplayModeString != null) {
				StringTokenizer tokenizer = new StringTokenizer(
						diplayModeString, ",");
				int width = Integer.parseInt(tokenizer.nextToken());
				int height = Integer.parseInt(tokenizer.nextToken());
				int bitDepth = Integer.parseInt(tokenizer.nextToken());
				int refreshRate = Integer.parseInt(tokenizer.nextToken());
				displayMode = new DisplayMode(width, height, bitDepth,
						refreshRate);
			}
			String tooltipsEnabledString = getProperty("tooltips", "true");
			tooltipsEnabled = Boolean.valueOf(tooltipsEnabledString)
					.booleanValue();

		} // try
		catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * save options to file
	 * 
	 */
	public void save() {
		FileOutputStream outputStream;
		try {
			outputStream = new FileOutputStream("fb.properties");
			setProperty("sound", String.valueOf(soundOn));
			setProperty("music", String.valueOf(musicOn));
			setProperty("language", language);
			setProperty("country", country);
			if (displayMode != null) {
				StringBuffer buff = new StringBuffer();
				buff.append(displayMode.getWidth());
				buff.append(',');
				buff.append(displayMode.getHeight());
				buff.append(',');
				buff.append(displayMode.getBitDepth());
				buff.append(',');
				buff.append(displayMode.getRefreshRate());
				setProperty("display", buff.toString());
			}
			setProperty("tooltips", String.valueOf(tooltipsEnabled));
			store(outputStream, "GAME OPTIONS");
		} // try
		catch (IOException e) {
			e.printStackTrace();
		}
	}

	/**
	 * Returns the country.
	 * 
	 * @return String
	 */
	public static String getCountry() {
		return getInstance().country;
	}

	/**
	 * Returns the language.
	 * 
	 * @return String
	 */
	public static String getLanguage() {
		return getInstance().language;
	}

	/**
	 * Returns the musicon.
	 * 
	 * @return boolean
	 */
	public static boolean isMusicOn() {
		return getInstance().musicOn;
	}

	public static boolean isTooltipsEnabled() {
		return getInstance().tooltipsEnabled;
	}

	/**
	 * Returns the soundon.
	 * 
	 * @return boolean
	 */
	public static boolean isSoundOn() {
		return getInstance().soundOn;
	}

	/**
	 * Sets the country.
	 * 
	 * @param country
	 *            The country to set
	 */
	public static void setCountry(String country) {
		getInstance().country = country;
	}

	/**
	 * Sets the language.
	 * 
	 * @param language
	 *            The language to set
	 */
	public static void setLanguage(String language) {
		getInstance().language = language;
	}

	/**
	 * Sets the musicon.
	 * 
	 * @param musicon
	 *            The musicon to set
	 */
	public static void setMusicOn(boolean musicon) {
		getInstance().musicOn = musicon;
	}

	/**
	 * Sets the soundon.
	 * 
	 * @param soundon
	 *            The soundon to set
	 */
	public static void setSoundOn(boolean soundon) {
		getInstance().soundOn = soundon;
	}

	/**
	 * @return Returns the displayMode.
	 */
	public static DisplayMode getDisplayMode() {
		return getInstance().displayMode;
	}

	/**
	 * @param displayMode
	 *            The displayMode to set.
	 */
	public static void setDisplayMode(DisplayMode displayMode) {
		getInstance().displayMode = displayMode;
	}

	/**
	 * @param tooltipsEnabled
	 *            The tooltipsEnabled to set.
	 */
	public static synchronized void setTooltipsEnabled(boolean tooltipsEnabled) {
		getInstance().tooltipsEnabled = tooltipsEnabled;
	}
}
