/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.phases;

import java.util.Vector;

import fb.army.entities.Courier;
import fb.army.entities.Entity;
import fb.army.entities.SummonedCreature;
import fb.army.entities.Unit;
import fb.main.Battle;

/**
 * phase for automatic movements
 * 
 * @author farfpeuf
 * 
 */
public class AutomaticMovementPhase extends SynchronizedPhase {
	/**
	 * phase name
	 */
	private static final String name = "automovephase";

	/**
	 * phase icon number
	 */
	private static final int iconNumber = 2;

	/**
	 * entities moving during this phase
	 */
	private Vector movers;

	/**
	 * constructor reset remaining move for all entities
	 * 
	 */
	public AutomaticMovementPhase() {
		super();
		movers = new Vector();
		Entity entity;
		for (int i = 0; !stop
				&& i < Battle.getInstance().getActiveEntities().size(); i++) {
			entity = (Entity) (Battle.getInstance().getActiveEntities().get(i));

			entity.resetRemainingMove();

			if (entity.getMagicShield() > 0) {
				entity.consumeMagicShield();
			}
		}
	}

	/**
	 * @return phase name
	 */
	public String getName() {
		return name;
	}

	/**
	 * @return phase icon number
	 */
	public int getIconNumber() {
		return iconNumber;
	}

	/**
	 * @return phase toolip
	 */
	public String getTooltip() {
		return null;// no tooltip
	}
	/**
	 * proceed to phase next step
	 */

	public void nextStep() {

		Entity entity, target;

		if (armies[current].isHandled()) {
			for (int i = 0; !stop
					&& i < armies[current].getActiveEntities().size(); i++) {
				entity = (Entity) (armies[current].getActiveEntities().get(i));
				if (entity instanceof SummonedCreature && entity.isActive()) {
					SummonedCreature summonedCreature = (SummonedCreature) entity;
					if (summonedCreature.getSummoner().isActive()) {
						if (!summonedCreature.drainMagic()) {
							summonedCreature.moveTo(summonedCreature
									.getSummoner());
							movers.add(entity);
						}
					} else {
						summonedCreature.bannish();
					}
				}
				if (entity.isCourier()) {
					if (entity.getOpposingContacts().size() == 0) {
						Courier courier = (Courier) entity;
						if (courier.getReceiver() != null) {
							courier.moveTo(courier.getReceiver());
							movers.add(entity);
						}
					}
				}
				if (entity.isUnit()) {
					Unit unit = (Unit) entity;
					if (entity.getState() == 'b' || unit.getOrder() == 'a') {
						if (entity.getOpposingContacts().size() == 0) {
							Entity closesttarget = null;
							int mindist = 9999, dist;
							for (int j = 0; j < Battle.getInstance()
									.getActiveEntities().size(); j++) {
								target = (Entity) Battle.getInstance()
										.getActiveEntities().get(j);
								if ((target.isEnnemy(entity))) {
									dist = entity.distanceTo(target);
									if (entity.moveClear(target)) {
										dist -= (entity.getMovement() * 8);
									}
									if (dist < mindist) {
										mindist = entity.distanceTo(target);
										closesttarget = target;
									}
								}
							}
							if (closesttarget != null) {
								entity.moveTo(closesttarget);
								movers.add(entity);
							}
						}
					}
				}
			}
			// end this army
			setToken(current);
			if (!finished()) {
				changeArmy();
			}
		} else { // not handled
			waitNetAction();
		}

	}

	/**
	 * end the phase
	 */
	public void end() {
		Entity entity;
		for (int i = 0; i < movers.size(); i++) {
			entity = (Entity) movers.get(i);
			// attendre la fin des mouvements
			while (!stop && entity.isWalking()) {
				sleep(125);
			}
		}
	}
}