/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.phases;

import java.awt.Point;
import java.awt.Rectangle;
import java.awt.geom.Area;
import java.util.Iterator;
import java.util.List;

import fb.army.entities.Entity;
import fb.artificialintelligence.TargetFinder;
import fb.battlefield.BattleFieldPanel;
import fb.gui.frames.BattleFrame;
import fb.main.FB;
import fb.main.Battle;
import fb.util.DX;
import fb.util.NetworkDX;

/**
 * armies deployment phase
 * 
 * @author farfpeuf
 * 
 */
public class DeploymentPhase extends BinaryPhase {
	/**
	 * phase name
	 */
	private static final String name = "deploymentphase";

	/**
	 * phase icon number
	 */
	private static final int iconNumber = 2;

	/**
	 * current deployment area
	 */
	private Area deployArea;

	/**
	 * entity destination x and y coordinate
	 */
	private int destX, destY;

	/**
	 * constructor
	 * 
	 */
	public DeploymentPhase() {
		super();
		deployArea = new Area();
		setWithPhasePanel(false);
	}

	/**
	 * @return phase name
	 */
	public String getName() {
		return name;
	}

	/**
	 * @return phase icon number
	 */
	public int getIconNumber() {
		return iconNumber;
	}

	/**
	 * set current actor destintion
	 * 
	 * @param destX
	 *            destination x coordinate
	 * @param destY
	 *            destination y coordinate
	 */
	public void setDestination(int destX, int destY) {

		int oldDestX = this.destX;
		int oldDestY = this.destY;
		this.destX = destX;
		this.destY = destY;
		Entity actor = getActor();
		if (actor != null) {
			if (oldDestX != destX || oldDestY != destY) {
				BattleFieldPanel.refreshIt(getActor().getDeploymentBounds(
						oldDestX, oldDestY));
				BattleFieldPanel.refreshIt(getActor().getDeploymentBounds(
						destX, destY));
			}
		}

	}

	/**
	 * set phase actor
	 * 
	 * @param actor
	 *            the actor
	 */
	public void setActor(Entity actor) {
		Entity oldActor = getActor();
		if (oldActor != null) {
			BattleFieldPanel.refreshIt(oldActor.getDeploymentBounds(destX,
					destY));
		}
		super.setActor(actor);

		if (actor != null) {
			BattleFieldPanel.refreshIt(actor.getDeploymentBounds(destX, destY));
		}
	}

	/**
	 * change current playing army
	 * 
	 */
	public void changeArmy() {
		int i = current + 1;
		if (i == armies.length) {
			i = 0;
		}
		while (Battle.getLoop().getPhaseTokens()[i] && !finished()) {
			i++;
			if (i == armies.length) {
				i = 0;
			}
		}
		current = i;
		// FB.debug("changeArmy " + armies[current].getPlayerName());
		if (!finished()) {
			Battle.getInstance().setArmy(armies[current]);
		} else {
			Battle.getInstance().setArmy(null);
		}
	}

	/**
	 * determine first playing army
	 * 
	 */
	public void initCurrent() {
		if (FB.isNet()) {
			current = NetworkDX.roll(armies.length) - 1;
		} else {
			current = DX.roll(armies.length) - 1;
		}
		Battle.getInstance().setArmy(armies[current]);
		// FB.debug("initCurrent " + armies[current].getPlayerName());

	}

	/**
	 * change current playing army and initialize actors selection
	 */
	public void nextArmy() {
		changeArmy();
		if (!finished()) {
			initActorSelection();
		}
	}

	/**
	 * proceed to phase next step
	 */
	public void nextStep() {

		if (armies[current].isHandled()) {
			if (!armies[current].isAi() && !Battle.getInstance().isAutoDeploy()) {
				handlePlayer();
			} else {
				setToken(current);
				if (!finished()) {
					nextArmy();
				}
			}
		} else {
			setActor(null);
			waitNetAction();
		}

	}

	/**
	 * handle human player action
	 * 
	 */
	public void handlePlayerAction() {
		if (getActor().canDeployTo(clickedPoint.x, clickedPoint.y, deployArea)) {
			action();
			initActorSelection();
		}
	}

	/**
	 * player Performed an Action
	 * 
	 */
	public void playerPerformedAction() {
		setWaitCursor();
		BattleFrame.getInstance().getNextPhaseButton().setEnabled(false);
		if (clickedPoint == null) {
			setToken(current);
			if (!finished()) {
				nextArmy();
			}
		} else {
			handlePlayerAction();
		}
	}

	/**
	 * handle human player
	 * 
	 */
	public void handlePlayer() {
		if (clickedPoint == null) {
			setToken(current);
			if (!finished()) {
				nextArmy();
			}
		} else {
			if (isActorSelection()) {

				if (getPotentialActors().size() > 0) {

					setActor((Entity) getPotentialActors().get(0));
					BattleFrame.getInstance().displayEntityCharacteristics(
							getActor());

					List rectangles = Battle.getInstance().getRectangles(
							armies[current]);
					if (FB.getInstance().isDebug()) {
						rectangles.add(Battle.getInstance().getInterior());
					}
					double initialAngle = getInitialAngle(
							(Rectangle) rectangles.get(0), Battle.getInstance()
									.getEnnemyRectangles(armies[current]));
					getActor().rotate(initialAngle);
					Iterator iter = rectangles.iterator();

					Area oldDeployArea = deployArea;
					deployArea = new Area();
					while (iter.hasNext()) {
						deployArea.add(new Area((Rectangle) iter.next()));
					}
					deployArea.subtract(getActor().getMovementObstacles());
					BattleFieldPanel.refreshIt(oldDeployArea.getBounds());
					BattleFieldPanel.refreshIt(deployArea.getBounds());
					setActorselection(false);
				} else {
					if (!finished()) {
						nextArmy();
					}
				}
			} else {
				waitForPlayerAction();
			}
		}
	}

	/**
	 * wait for player action
	 * 
	 */
	protected void waitForPlayerAction() {
		BattleFrame.getInstance().getNextPhaseButton().setEnabled(true);
		if (isActorSelection()) {
			setDefaultCursor();
		} else {
			setTargetCursor();
		}
		waitingForPlayerAction = true;
	}

	/**
	 * @return AI thread that handles choice of the best target among a set of
	 *         availables targets
	 */
	public TargetFinder getTargetFinder() {
		return null;// not used

	}

	/**
	 * calculate available actors
	 * 
	 */
	public void calculatePotentialActors() {
		Entity entity;
		clearPotentialActors();
		for (int i = 0; i < armies[current].getActiveEntities().size(); i++) {
			entity = (Entity) (armies[current].getActiveEntities().get(i));
			if (isActor(entity)) {
				addPotentialActor(entity);
			}
		}
		setActorselection(true);
		if (getPotentialActors().size() == 0) {
			setToken(current);
			if (!finished()) {
				nextArmy();
			}
		} else {
			clickedPoint = new Point();
		}
	}

	/**
	 * is this entity a potential target for specified actor
	 * 
	 * @param a
	 *            phase actor
	 * @param e
	 *            phase target
	 * @return true if this entity is a potential target for specified actor
	 */
	public boolean isTarget(Entity actor, Entity e) {
		return false;
	}

	/**
	 * is this entity a potential target for current actor
	 * 
	 * @param e
	 *            phase target
	 * @return true if this entity is a potential target for current actor
	 */
	public boolean isTarget(Entity e) {
		return false;
	}

	/**
	 * is this entity a potential actor for this phase
	 * 
	 * @param a
	 *            phase actor
	 * @return true if this entity is a potential actor
	 */
	public boolean isActor(Entity e) {
		return !e.isDeployed();
	}

	/**
	 * proceed to phase action
	 * 
	 */
	public void action() {
		getActor().deployTo(clickedPoint.x, clickedPoint.y);
	}

	/**
	 * calculate phase results
	 * 
	 */
	public void calculateResults() {
		setActor(null);
		Battle.getInstance().deployArmies();
		BattleFieldPanel.refreshIt(deployArea.getBounds());
	}

	/**
	 * caclculate army initial direction
	 * 
	 * @param rect
	 *            army deployment area
	 * @param ennemyRectangles
	 *            ennemies deployment areas
	 * @return army initial direction
	 */
	public static double getInitialAngle(Rectangle rect, List ennemyRectangles) {
		int minDist = Integer.MAX_VALUE, dist;
		double dx, dy;
		double initialAngle = 0.0;
		double deupi = 2 * Math.PI, theta;
		Rectangle ennemyRect;
		for (int i = 0; i < ennemyRectangles.size(); i++) {
			ennemyRect = (Rectangle) ennemyRectangles.get(i);
			dx = ennemyRect.getCenterX() - rect.getCenterX();
			dy = ennemyRect.getCenterY() - rect.getCenterY();
			dist = (int) java.lang.Math.sqrt(dx * dx + dy * dy);
			if (dist < minDist) {
				minDist = dist;
				theta = -Math.atan2(dy, dx);
				theta -= Math.PI / 2;
				initialAngle = theta % deupi;
			}
		}
		if (initialAngle < -5.8905) {
			return 0;
		} else if (initialAngle < -4.3197) {
			return Math.PI / 2;
		} else if (initialAngle < -2.7489) {
			return Math.PI;
		} else if (initialAngle < -1.1781) {
			return 3 * Math.PI / 2;
		} else if (initialAngle < 0.3927) {
			return 0;
		} else if (initialAngle < 1.9635) {
			return Math.PI / 2;
		} else if (initialAngle < 3.5343) {
			return Math.PI;
		} else if (initialAngle < 5.1051) {
			return 3 * Math.PI / 2;
		} else {
			return 0;
		}
	}

	/**
	 * @return Returns the deployArea.
	 */
	public Area getDeployArea() {
		return deployArea;
	}
}