/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.spells;

import javax.swing.ImageIcon;

import fb.army.entities.Entity;
import fb.army.entities.Individual;
import fb.army.entities.SpellCaster;
import fb.army.hierarchy.Army;
import fb.battlefieldelements.MagicEffect;
import fb.main.FB;
import fb.main.Battle;
import fb.util.D6;
import fb.util.Resources;

/**
 * Fires an ennergy ball to the ennemy individual
 * 
 * @author farfpeuf
 * 
 */
public class EnergyBall extends AttackSpell {
	/**
	 * spell visual effects
	 */
	private static final ImageIcon[] effects = new ImageIcon[] { Resources
			.getSign("energyball") };

	/**
	 * constructor
	 * 
	 */
	public EnergyBall() {
		super();
		name = "energyball";
	}

	/**
	 * is this entity a potential target for specified actor
	 * 
	 * @param a
	 *            spell actor
	 * @param e
	 *            spell target
	 * @return true if this entity is a potential target for specified actor
	 */
	public boolean isTarget(Entity a, Entity e) {
		return e.isIndividual() && a.lineOfSightTo(e) < 9999
				&& super.isTarget(a, e) && e.isEnnemy(a);
	}

	/**
	 * is this entity a potential target for current actor
	 * 
	 * @param e
	 *            spell target
	 * @return true if this entity is a potential target for current spell
	 *         caster
	 */
	public boolean isTarget(Entity e) {
		return e.isIndividual() && caster.lineOfSightTo(e) < 9999
				&& super.isTarget(e) && e.isEnnemy(caster);
	}

	/**
	 * cast the spell
	 * 
	 * @return true if the spell was successfull
	 */
	public boolean castSpell() {

		if (caster.spendMagicPoints(cost())) {
			int kills = 0;

			for (int i = 0; i < 2; i++) {
				int d6 = D6.roll();
				if (d6 >= 5) {
					kills++;
				}
			}

			target.wound(kills);

			return true;

		}

		return false;

	}

	/**
	 * 
	 * @param caster
	 *            spell caster
	 * @param target
	 *            spell target
	 * @return spell cost
	 */
	public int cost(SpellCaster caster, Entity target) {
		return D6.roll() + D6.roll() + (int) (caster.stepsTo(target) / 10);
	}

	/**
	 * 
	 * @param caster
	 *            spell caster
	 * @param target
	 *            spell target
	 * @return spell maximum cost
	 */
	public int maxCost(SpellCaster caster, Entity target) {
		return 12 + (int) (caster.stepsTo(target) / 10);
	}

	/**
	 * 
	 * @param caster
	 *            spell caster
	 * @param target
	 *            spell target
	 * @return spell minimumu cost
	 */
	public int minCost(SpellCaster caster, Entity target) {
		return 2 + (int) (caster.stepsTo(target) / 10);
	}

	/**
	 * 
	 * @param caster
	 *            spell caster
	 * @param target
	 *            spell target
	 * @return average cost
	 */
	public int averageCost(SpellCaster caster, Entity target) {
		return 7 + (int) (caster.stepsTo(target) / 10);
	}

	/**
	 * prepare spell visual effects
	 * 
	 */
	public void prepareEffects() {
		caster.getMagics().add(
				new MagicEffect(caster, ((Individual) target).getModel(),
						effects));

	}

	/**
	 * end the spell
	 * 
	 * @throws InterruptedException
	 */
	public void endSpell() throws InterruptedException {

		if (((Individual) target).isDead() && FB.getInstance().isGraphicMode()) {
			((Individual) target).getModel().setAction('d');
			((Individual) target).getModel().setAnimationCounter(1);

		}
		while (target.isDying()) {
			Thread.sleep(125);

		}

		target.commitKill();
		magicSynchronize();
		Army army;
		for (int i = 0; i < Battle.getInstance().getArmies().length; i++) {
			army = Battle.getInstance().getArmies()[i];
			if (army.isHandled() && army.mustTestMorale()) {
				army.moraleTest();
			}
		}
		magicSynchronize();
		Entity entity;

		for (int i = 0; i < Battle.getInstance().getActiveEntities().size(); i++) {
			entity = (Entity) Battle.getInstance().getActiveEntities().get(i);

			// attendre la fin des mouvements
			while (entity.isWalking()) {
				Thread.sleep(125);
			}
			if (entity.getState() == 'r') {
				entity.commitRout();
			}
		}

	}

	/**
	 * @param potentialTarget
	 *            the target
	 * @return weight modifiers for artificial intelligence for specified target
	 */
	public int getWeight(Entity potentialTarget) {
		int weight = super.getWeight(potentialTarget);

		if (potentialTarget.getTotalLifePoints() == 1) {
			weight += 2;
		}
		return weight;
	}
}
