/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.spells;

import fb.army.entities.Entity;
import fb.army.entities.SpellCaster;
import fb.util.D6;

/**
 * Divides permanently by 2 the size, strength, resistance, attack and movement
 * of the ennemy target
 * 
 * @author farfpeuf
 * 
 */
public class Shrink extends AttackSpell {

	/**
	 * farfpeuf
	 * 
	 */
	public Shrink() {
		super();
		name = "shrink";
	}

	/**
	 * is this entity a potential target for specified actor
	 * 
	 * @param a
	 *            spell actor
	 * @param e
	 *            spell target
	 * @return true if this entity is a potential target for specified actor
	 */
	public boolean isTarget(Entity a, Entity e) {
		return e.getScale() > 0.25 && e.isEnnemy(a)
				&& a.lineOfSightTo(e) < 9999 && super.isTarget(a, e);
	}

	/**
	 * is this entity a potential target for current actor
	 * 
	 * @param e
	 *            spell target
	 * @return true if this entity is a potential target for current spell
	 *         caster
	 */
	public boolean isTarget(Entity e) {
		return e.getScale() > 0.25 && e.isEnnemy(caster)
				&& caster.lineOfSightTo(e) < 9999 && super.isTarget(e);
	}

	/**
	 * cast the spell
	 * 
	 * @return true if the spell was successfull
	 */
	public boolean castSpell() {
		if (caster.spendMagicPoints(cost())) {
			return true;
		}

		return false;

	}

	/**
	 * 
	 * @param caster
	 *            spell caster
	 * @param target
	 *            spell target
	 * @return spell cost
	 */
	public int cost(SpellCaster caster, Entity target) {
		int cost = target.totalWorth() + 2 * caster.stepsTo(target);
		int d6 = D6.roll();
		if (d6 <= 2) {
			cost = (int) (cost / 2);
		} else if (d6 >= 5) {
			cost = cost * 2;
		}
		return cost;
	}

	/**
	 * 
	 * @param caster
	 *            spell caster
	 * @param target
	 *            spell target
	 * @return spell minimumu cost
	 */
	public int minCost(SpellCaster caster, Entity target) {
		return (int) ((target.totalWorth() + 2 * caster.stepsTo(target)) / 2);
	}

	/**
	 * 
	 * @param caster
	 *            spell caster
	 * @param target
	 *            spell target
	 * @return spell maximum cost
	 */
	public int maxCost(SpellCaster caster, Entity target) {
		return (target.totalWorth() + 2 * caster.stepsTo(target)) * 2;
	}

	/**
	 * 
	 * @param caster
	 *            spell caster
	 * @param target
	 *            spell target
	 * @return average cost
	 */
	public int averageCost(SpellCaster caster, Entity target) {
		return (int) ((target.totalWorth() + 2 * caster.stepsTo(target)) * 3.5 / 3.0);
	}

	/**
	 * end the spell
	 * 
	 * @throws InterruptedException
	 */
	public void endSpell() throws InterruptedException {
		target.shrink();
		while (target.isWalking()) {
			Thread.sleep(125);
		}
		magicSynchronize();
	}

	/**
	 * @param potentialTarget
	 *            the target
	 * @return weight modifiers for artificial intelligence for specified target
	 */
	public int getWeight(Entity potentialTarget) {
		int weight = super.getWeight(potentialTarget) - 3;

		if (potentialTarget.getStrength() > 1) {
			weight += 1;
		}

		if (potentialTarget.getResistance() > 1) {
			weight += 1;
		}

		if (potentialTarget.getWorth() > 1) {
			weight += 1;
		}

		if (potentialTarget.getMovement() > 8) {
			weight += 1;
		}
		return weight;
	}
}
