/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.spells;

import fb.army.entities.Entity;
import fb.army.entities.SpellCaster;

/**
 * a spell with a friendly or ennemy target entity
 * 
 * @author farfpeuf
 * 
 */
public abstract class TargetSpell extends BasicSpell {
	/**
	 * the spell target
	 */
	protected Entity target;

	/**
	 * @return string to send spell to remote computers
	 */
	protected String spellString() {
		return super.spellString() + "#" + target.getId();
	}

	/**
	 * @param target
	 *            The target to set.
	 */
	public void setTarget(Entity target) {
		this.target = target;
	}

	/**
	 * 
	 * @param caster
	 *            spell caster
	 * @param target
	 *            spell target
	 * @return spell cost
	 */
	public abstract int cost(SpellCaster caster, Entity target);

	/**
	 * 
	 * @param caster
	 *            spell caster
	 * @param target
	 *            spell target
	 * @return spell maximum cost
	 */
	public abstract int maxCost(SpellCaster caster, Entity target);

	/**
	 * 
	 * @param caster
	 *            spell caster
	 * @param target
	 *            spell target
	 * @return spell minimumu cost
	 */
	public abstract int minCost(SpellCaster caster, Entity target);

	/**
	 * 
	 * @param caster
	 *            spell caster
	 * @param target
	 *            spell target
	 * @return average cost
	 */
	public abstract int averageCost(SpellCaster caster, Entity target);

	/**
	 * @return spell cost
	 */
	public int cost() {
		return cost(this.caster, this.target);
	}

	/**
	 * @return spell maximum cost
	 */
	public int maxCost() {
		return maxCost(this.caster, this.target);
	}

	/**
	 * @return spell minimum cost
	 */
	public int minCost() {
		return minCost(this.caster, this.target);
	}

	/**
	 * @return spell average cost
	 */
	public int averageCost() {
		return averageCost(this.caster, this.target);
	}

	/**
	 * is this entity a potential target for specified actor
	 * 
	 * @param a
	 *            spell actor
	 * @param e
	 *            spell target
	 * @return true if this entity is a potential target for specified actor
	 */
	public boolean isTarget(Entity a, Entity e) {
		return e.getMagicShield() == 0
				&& ((SpellCaster) a).getMagicPoints() >= minCost(
						(SpellCaster) a, e);
	}

	/**
	 * is this entity a potential target for current actor
	 * 
	 * @param e
	 *            spell target
	 * @return true if this entity is a potential target for current spell
	 *         caster
	 */
	public boolean isTarget(Entity e) {
		return e.getMagicShield() == 0
				&& this.caster.getMagicPoints() >= minCost(this.caster, e);
	}

	/**
	 * @param potentialTarget
	 *            the target
	 * @return weight modifiers for artificial intelligence for specified target
	 */
	public int getWeight(Entity potentialTarget) {
		int weight = caster.getMagicPoints() / 20;
		int maxcost = maxCost(caster, potentialTarget);
		if (caster.getMagicPoints() < averageCost(caster, potentialTarget)) {
			weight -= 2;
		}
		if (caster.getMagicPoints() < maxcost) {
			weight -= 1;
		}
		if (caster.getMagicPoints() > 2 * maxcost) {
			weight += 2;
		}
		if (caster.getMagicPoints() > 3 * maxcost) {
			weight += 1;
		}
		if (caster.getMagicPoints() > 4 * maxcost) {
			weight += 1;
		}

		if (caster.getArmy().degradation() > 2) {
			weight += 2;
		}
		if (caster.getTotalLifePoints() == 1) {
			weight += 1;
		}
		if (caster.getOpposingContacts().size() > 0) {
			weight += 1;
		}
		return weight;
	}

	/**
	 * @return true if AI plyer should cast this spell
	 */
	public boolean shouldAICast() {
		return true;
	}

}
