/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.spells;

import java.util.Iterator;

import javax.swing.ImageIcon;

import fb.army.entities.Entity;
import fb.army.entities.SpellCaster;
import fb.army.entities.Unit;
import fb.battlefieldelements.MagicEffect;
import fb.battlefieldelements.Warrior;
import fb.util.D6;
import fb.util.Resources;

/**
 * Provokes panic among the ennemy unit. The unit loses 3 morale points
 * 
 * @author farfpeuf
 * 
 */
public class Terror extends AttackSpell {
	/**
	 * spell visual effects
	 */
	private static final ImageIcon[] effects = new ImageIcon[] { Resources
			.getSign("skull") };

	/**
	 * constructor
	 * 
	 */
	public Terror() {
		super();
		name = "terror";
	}

	/**
	 * is this entity a potential target for specified actor
	 * 
	 * @param a
	 *            spell actor
	 * @param e
	 *            spell target
	 * @return true if this entity is a potential target for specified actor
	 */
	public boolean isTarget(Entity a, Entity e) {
		return e.isUnit() && e.isEnnemy(a) && a.lineOfSightTo(e) < 9999
				&& super.isTarget(a, e);
	}

	/**
	 * is this entity a potential target for current actor
	 * 
	 * @param e
	 *            spell target
	 * @return true if this entity is a potential target for current spell
	 *         caster
	 */
	public boolean isTarget(Entity e) {
		return e.isUnit() && e.isEnnemy(caster)
				&& caster.lineOfSightTo(e) < 9999 && super.isTarget(e);
	}

	/**
	 * cast the spell
	 * 
	 * @return true if the spell was successfull
	 */
	public boolean castSpell() {

		if (caster.spendMagicPoints(cost())) {
			return true;
		}

		return false;

	}

	/**
	 * 
	 * @param caster
	 *            spell caster
	 * @param target
	 *            spell target
	 * @return spell cost
	 */
	public int cost(SpellCaster caster, Entity target) {
		int cost = ((Unit) target).totalWorth() + 2 * caster.stepsTo(target);
		int d6 = D6.roll();
		if (d6 <= 2) {
			cost = (int) (cost / 2);
		} else if (d6 >= 5) {
			cost = cost * 2;
		}
		return cost;
	}

	/**
	 * 
	 * @param caster
	 *            spell caster
	 * @param target
	 *            spell target
	 * @return spell minimumu cost
	 */
	public int minCost(SpellCaster caster, Entity target) {
		return (int) ((((Unit) target).totalWorth() + 2 * caster
				.stepsTo(target)) / 2);
	}

	/**
	 * 
	 * @param caster
	 *            spell caster
	 * @param target
	 *            spell target
	 * @return spell maximum cost
	 */
	public int maxCost(SpellCaster caster, Entity target) {
		return (((Unit) target).totalWorth() + 2 * caster.stepsTo(target)) * 2;
	}

	/**
	 * 
	 * @param caster
	 *            spell caster
	 * @param target
	 *            spell target
	 * @return average cost
	 */
	public int averageCost(SpellCaster caster, Entity target) {
		return (int) ((((Unit) target).totalWorth() + 2 * caster
				.stepsTo(target)) * 3.5 / 3.0);
	}

	/**
	 * prepare spell visual effects
	 * 
	 */
	public void prepareEffects() {
		Iterator it = target.getModels().iterator();
		while (it.hasNext()) {
			caster.getMagics().add(
					new MagicEffect(caster, (Warrior) it.next(), effects));
		}
	}

	/**
	 * end the spell
	 * 
	 * @throws InterruptedException
	 */
	public void endSpell() throws InterruptedException {

		if (target.isHandled()) {
			((Unit) target).magicThreat();
		}
		magicSynchronize();
		while (target.isWalking()) {
			Thread.sleep(125);
		}
		if (target.getState() == 'r') {
			target.commitRout();
		}
	}

	/**
	 * @param potentialTarget
	 *            the target
	 * @return weight modifiers for artificial intelligence for specified target
	 */
	public int getWeight(Entity potentialTarget) {
		int weight = super.getWeight(potentialTarget);

		if (((Unit) potentialTarget).getMorale() < 4) {
			weight += 2;
		}
		return weight;
	}
}
