/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.util;

import java.lang.reflect.Constructor;

import fb.army.entities.Entity;
import fb.main.LoadingException;
/**
 * factory to create entities from a line read in the catalog
 * @author farfpeuf
 *
 */
public class EntityFactory {

	/**
	 * entity classes package prefix
	 */
	private static final String ENTITY_PACKAGE_PREFIX = "fb.army.entities.";

	/**
	 * build entity from line read in catalog
	 * @param lineRead the line read
	 * @param linenumber the line number
	 * @return the corresponding entity
	 * @throws LoadingException
	 */
	public static Entity buildEntity(String lineRead, int linenumber)
			throws LoadingException {

		int index = lineRead.indexOf('\t');
		String entitytype = lineRead.substring(0, index);
		String entitystring = lineRead.substring(index + 1);

		// get entity constructor and use it
		try {
			Constructor constructor = Class.forName(
					ENTITY_PACKAGE_PREFIX + entitytype).getConstructor(
					new Class[] { String.class, Integer.TYPE });
			return (Entity) constructor.newInstance(new Object[] {
					entitystring, new Integer(linenumber) });
		} catch (Exception e) {
			throw new LoadingException(e);
		}
	}

}
