/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.util;

import java.util.Random;

/**
 * an network-synchronized X-sided dice
 * Every remote computer gets the same result when rolling it
 * 
 * @author farfpeuf
 * 
 */
public class NetworkDX {
	/**
	 * the random number generator instance
	 */
	private static Random randomNumberGenerator;

	/**
	 * roll the dice
	 * 
	 * @param x
	 *            the number of sides
	 * @return the dice score
	 */
	public static int roll(int x) {
		if (x > 0) {
			return randomNumberGenerator.nextInt(x) + 1;
		} else {
			return 0;
		}
	}

	/**
	 * initialize the random number generator
	 * 
	 * @param seed
	 *            the random seed which is the same for every remote computer
	 */
	public static synchronized void initRNG(long seed) {
		if (randomNumberGenerator == null) {
			randomNumberGenerator = new Random(seed);
		}
	}

}
